# 1. 初期設定パート ******************************************************

# モジュール読み込み ******************************************************
import time              # 時間に関するPython標準モジュールです。
import RPi.GPIO as GPIO  # Raspberry PiのGPIO（汎用入出力）を制御するためのPythonモジュールです。
import os                # os機能に関するPython標準モジュールです。
import subprocess        # プロセス管理に関するPython標準モジュールです。
import cv2               # OpenCVのPythonモジュールです。
import socket            # socket通信に関するPython標準モジュールです。
import threading         # マルチスレッド処理に関するPython標準モジュールです。
import pandas            # データの処理及び分析を用意にするPythonモジュールです。
import csv               # CSVデータ読み書きに使用するPython標準モジュールです。
from Adafruit_BME280 import *  # センサメーカー(Adafruit)がGitHub（https://github.com/adafruit/Adafruit_Python_BME280）に公開しているPythonモジュールです。
from grove_gesture_sensor import *  # GitHub（https://github.com/DexterInd/GrovePi/tree/master/Software/Python/grove_gesture_sensor）に公開されているPythonモジュールです。
from upspackv2 import *  # GitHub（https://github.com/rcdrones/UPSPACK_V3/）に公開されているPythonモジュールです。
import datetime          # 日付と時刻に関するPython標準モジュールです。
import requests          # HTTPリクエストに関するPythonモジュールです。
import wikipedia         # ウィキペディア検索用Pythonモジュールです。
from flask import (Flask, render_template, request, redirect, session, send_from_directory, Response)  # FlaskのPythonモジュールです。
from gtts import gTTS    # ”google text to speech”のPythonモジュールです。
import random            # 乱数生成に使用します。

# SIROの行動制御 *********************************************************

mode = 1
mode_string = ["テスト", "のんびり", "聞き耳", "ラジオ", "ラジコン", "ライブカメラ", "＊＊＊", "＊＊＊", "＊＊＊", "＊＊＊"]
# SIROの行動をモードで制御します。
# モードはまだ５つしかありませんが今後追加していく予定です。

g_parameter = [0, 0, 0, 0, 0]
# SIROの行動制御に汎用的に使用するパラメータです。

siro_silent = 1  # おしゃべり禁止
siro_dontmove = 1  # お散歩禁止
# テスト時などSIROの行動を制限したい場合に使用するパラメータです。
# ２つしかないですが今後追加していく予定です。

communication = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
# 家族とのコニュニケーションの進み具合を記憶するためのパラメータです。
# IDNo.ごとに記憶します。

thread_loop = 1  # サブスレッド終了制御フラグ
# サブスレッドの終了を制御するためのフラグです。

list_week = ['月曜日', '火曜日', '水曜日', '木曜日', '金曜日', '土曜日', '日曜日']
# 曜日の認識に使用します。

# GPIOの設定 *************************************************************

GPIO.setmode(GPIO.BCM)
# GPIOを使用する際にポート番号で指定するかピン番号で指定するかを設定しています。
# 例えばGPIO2はポート番号で指定すると”２”、ピン番号で指定すると”３”になります。
# ここではポート番号で指定するモードを選択しています。

# 基本機能1 移動する DCモータ制御 モータドライバ DRV8835　********************

MOTOR_B_IN1 = 12  # モータ左　ENABLE
MOTOR_B_IN2 = 16  # モータ左　PHASE
MOTOR_A_IN1 = 20  # モータ右　ENABLE
MOTOR_A_IN2 = 21  # モータ右　PHASE
# モータドライバDRV8835の接続先とGPIOのポート番号を紐づけています。

GPIO.setup(MOTOR_A_IN1, GPIO.OUT)
GPIO.setup(MOTOR_A_IN2, GPIO.OUT)
GPIO.setup(MOTOR_B_IN1, GPIO.OUT)
GPIO.setup(MOTOR_B_IN2, GPIO.OUT)
# GPIOを入力、出力どちらで使用するかを設定しています。ここではすべて出力に設定しています。

MOTOR_A_PWM = GPIO.PWM(MOTOR_A_IN1, 50)
MOTOR_B_PWM = GPIO.PWM(MOTOR_B_IN1, 50)
# GPIOの12と20をPWM出力に設定しています。
# パルス周期はとりあえず50Hzに設定しています。

siro_motor_speed = 50  # Duty比
# PWMのDuty比(0～100)でモーター印加電圧を変化させ回転速度を制御します。
# とりあえず50に設定しています。


def siro_motor(command):
    # SIROを走行させるための関数です。
    # 引数commandに従い左右のモータの回転速度、回転方向を制御します。

    if command == "go":  # まえ
        if not uss_alert:  # 障害物アラートフラグが立っている場合は前進を禁止します。
            MOTOR_A_PWM.start(siro_motor_speed)
            MOTOR_B_PWM.start(siro_motor_speed)
            GPIO.output(MOTOR_A_IN2, GPIO.LOW)
            GPIO.output(MOTOR_B_IN2, GPIO.LOW)
    elif command == "back":  # うしろ
        MOTOR_A_PWM.start(siro_motor_speed)
        MOTOR_B_PWM.start(siro_motor_speed)
        GPIO.output(MOTOR_A_IN2, GPIO.HIGH)
        GPIO.output(MOTOR_B_IN2, GPIO.HIGH)
    elif command == "right":  # 右回転
        MOTOR_A_PWM.start(siro_motor_speed)
        MOTOR_B_PWM.start(siro_motor_speed)
        GPIO.output(MOTOR_A_IN2, GPIO.HIGH)
        GPIO.output(MOTOR_B_IN2, GPIO.LOW)
    elif command == "left":  # 左回転
        MOTOR_A_PWM.start(siro_motor_speed)
        MOTOR_B_PWM.start(siro_motor_speed)
        GPIO.output(MOTOR_A_IN2, GPIO.LOW)
        GPIO.output(MOTOR_B_IN2, GPIO.HIGH)
    elif command == "stop":  # 停止
        MOTOR_A_PWM.stop()
        MOTOR_B_PWM.stop()
        GPIO.output(MOTOR_A_IN2, GPIO.LOW)
        GPIO.output(MOTOR_B_IN2, GPIO.LOW)


# 基本機能2 発話する OpenJtalk ********************************************

sp_vol = 50
# スピーカー音量を調整するためのパラメータです。50~100で調整します。

speaking = 0  # 発話中フラグ
# 発話中、サウンド再生中はフラグを立てます。発話中フラグが立っている間は音声認識結果をキャンセルします。


def siro_speak(serif):
    # 発話用の関数です。引数serif（文字列）を音声合成して発話します。

    global sp_vol, speaking
    # グローバル宣言しています。
    # 参照だけなら本当はいらないのですが筆者はとりあえず宣言しておくことにしています。

    cmd = "amixer cset numid=3 "+str(sp_vol)+"%"
    subprocess.call(cmd.split())
    # amixer（標準コマンド）を使ってスピーカー音量を設定しています
    # subprocess.callでコマンドを実行しています。

    # voice = " -m /usr/share/hts-voice/nitech-jp-atr503-m001/nitech_jp_atr503_m001.htsvoice" # 音声ファイル 男性の声
    voice = " -m /usr/share/hts-voice/mei/mei_normal.htsvoice"  # 音声ファイル 女性の声
    dic = " -x /var/lib/mecab/dic/open-jtalk/naist-jdic"  # 辞書ファイル
    # 使用する音声ファイルと辞書ファイルを指定しています。

    option = " -r 0.8 -fm 5"
    # OpenJtalkの音声パラメータを設定しています。指定しない場合はデフォルト値になるようです。
    # -a   オールパス値/声の質が変わります。　
    #      小さいとキテレツ大百科のコロスケみたいな声になります。大きいとおっさんみたいな声になります。1.0の前後で調整します。
    # -b   ポストフィルター係数/ 使い方がよくわかりませんでした。
    # -r   スピーチ速度係数/ 話すスピードが変わります。1.0がデフォルトなのでその前後で調整します。
    # -fm  追加ハーフトーン/ 声の高さが変わります。0.0 がデフォルトです。-15~15で調整しました。
    # -u   有声/無声境界値/ 使い方がよくわかりませんでした。
    # -jm  スペクトラム系列内変動の重み/ 抑揚が変わります。
    #      小さくするとロボットみたいな感じに、大きくするとミニオンみたいな声になります。1.0がデフォルトで0.5〜5.0で調整します。
    # -jf  FO系列内変動の重み/ 音量が変わります。0.5〜2.0で調整します。

    cmd = "echo " + serif + " | open_jtalk" + dic + voice + option + " -ow /tmp/siro_openjtalk.wav"
    os.system(cmd)
    # 文字列serifから音声ファイルsiro_OpenJtalk.wavを生成しています。
    # 音声ファイルは/tmp に保存し発話の都度ファイルを更新します。
    # 文字列serifを標準入力でOpen_Jtalkに入力するのにechoコマンドとパイプ（|）を使用しています。
    # os.systemを使ってコマンドを実行しています。

    speaking = 1
    # 発話中は発話中フラグを立てます。この間は音声認識結果をキャンセルします。

    cmd = "aplay /tmp/siro_openjtalk.wav"
    subprocess.call(cmd.split())
    # Open_Jtalkで生成した音声ファイルsiro_OpenJtalk.wavをaplay(標準コマンド)で再生しています。

    speaking = 0
    # 発話が終了したら発話中フラグを下げます。


# 基本機能3 サウンド再生　**************************************************

def siro_sound(sound):
    # サウンド再生用の関数です。
    # 引数soundで音源ファイルを選択して再生します。
    # 音源ファイルはmp3で、mpg321というソフトを使用しています。

    global sp_vol, speaking

    speaking = 1
    # サウンド再生中は発話中フラグを立てます。この間は音声認識結果をキャンセルします。

    if sound == "ごきげん":
        # 引数soundで再生する音源を指定します。

        cmd = "amixer cset numid=3 "+str(sp_vol*0.8)+"%"
        subprocess.call(cmd.split())
        # amixer（標準コマンド）を使ってスピーカー音量を設定しています。
        # パラメータsp_volに係数をかけて音源毎に音量を微調整しています。ここでは0.6掛けしています。
        # subprocess.callでコマンドを実行しています。

        cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_1.mp3"
        subprocess.call(cmd.split())
        # 音声ファイルを指定してmpg321で再生しています。
        # subprocess.callでコマンドを実行しています。

    # 以下、同様に音源を追加します。
    elif sound == "タイマー":
        cmd = "amixer cset numid=3 "+str(sp_vol*1.0)+"%"
        subprocess.call(cmd.split())
        cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/alerm.mp3"
        subprocess.call(cmd.split())
    elif sound == "ピー":
        cmd = "amixer cset numid=3 "+str(sp_vol*0.6)+"%"
        subprocess.call(cmd.split())
        cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/pi.mp3"
        subprocess.call(cmd.split())
    elif sound == "はと":
        cmd = "amixer cset numid=3 "+str(sp_vol*0.9)+"%"
        subprocess.call(cmd.split())
        cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/hatodokei.mp3"
        subprocess.call(cmd.split())
    elif sound == "メッセージ消去":
        cmd = "amixer cset numid=3 "+str(sp_vol*0.6)+"%"
        subprocess.call(cmd.split())
        cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/vanish.mp3"
        subprocess.call(cmd.split())
    elif sound == "カメラ":
        cmd = "amixer cset numid=3 "+str(sp_vol*0.9)+"%"
        subprocess.call(cmd.split())
        cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/camera.mp3"
        subprocess.call(cmd.split())

    speaking = 0
    # サウンド再生が終了したら発話中フラグを下げます。

    # 以下は未使用です。
    # cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_1.mp3"   # ごきげんにおどけた感じ
    # cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_2.mp3"  # ごきげんに何か訴える感じ
    # cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_3.mp3" #  なにかを完了させた感じ
    # cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_4.mp3"  # 困った感じ
    # cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_5.mp3"  # ちょっと興奮した感じ
    # cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_6.mp3" # 怒って訴える感じ
    # cmd = "mpg321 /home/kuro/kurosSIRO/data/sound/R2D2_7.mp3" # # ガンパっている感じ


# 基本機能4 顔認識 ********************************************************

face_cascade = \
    cv2.CascadeClassifier('/home/kuro/kurosSIRO/haarcascades/haarcascade_frontalface_alt.xml')
# 顔認識学習モデル（カスケード分類器）をファイルを指定して読み込んでいます。

cap = cv2.VideoCapture(0)
# クラスのインスタンス生成

SCREEN_WIDTH = 640  # 画像サイズ WIDTH
SCREEN_HEIGHT = 480  # 画像サイズ HEIGHT
cap.set(3, SCREEN_WIDTH)
cap.set(4, SCREEN_HEIGHT)
# カメラの解像度を設定しています。
# 解像度を大きくすると画像が鮮明になりますが処理速度が遅くなります。

cap.set(cv2.CAP_PROP_FPS, 30)
# FPSを設定しています。
# 使用したUSBカメラはFPS30で固定なので設定してもあまり意味がないのですが一応明示しています。

cap.set(cv2.CAP_PROP_BUFFERSIZE, 1)
# バッファサイズを最小の１に設定しています。
# こうすることでバッファ分のフレームの遅延がなくなります。

minW = 50
minH = 50
# 認識可能な顔のサイズを設定しています。
# 大きくすると遠くの顔が認識できなくなりますが誤認識が少なくなります。
# 逆に小さくすると遠くの顔も認識できる様になりますが壁の模様を顔と間違える等の誤認識が増えます。

recognizer = cv2.face.LBPHFaceRecognizer_create()
# 顔判別クラスのインスタンス生成

recognizer.read('/home/kuro/kurosSIRO/trainer.yml')
# 作成した顔判別機械学習モデルを読み込んでいます。

font = cv2.FONT_HERSHEY_SIMPLEX
# 画像に文字を記載する際のフォントを指定しています。

face_counter = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
# 顔判別カウンターです。
# 判別した顔のIDNo.をカウントアップしていきます。
# ID番号は 0: 自分　1：妻　2：長男　3：次男　9:不審者　としています。 IDNo.4~8は未割り当てです。

face_position = [[0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0]]
# 顔を検出したら顔の位置とサイズを求め、位置関係（左右前後）の把握に使用します。

# ret, frame = cap.read()


def siro_facerecog():
    # 顔認識処理を行う関数です。

    global thread_loop, face_counter, face_position

    while thread_loop:
        # メイン処理終了時にthread_loopを0にしてループから脱出させてサブスレッドを終了させます。

        ret, frame = cap.read()
        # カメラ画像を取得しています。

        grayimg = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        # 顔認識を行う前に画像をモノクロ処理して情報量を減らします。

        facerect = face_cascade.detectMultiScale(grayimg, minSize=(minW, minH))
        # 顔認識を行っています。

        if len(facerect) > 0:
            # 顔を見つけたら

            for (x, y, w, h) in facerect:
                # 顔を一つ一つ確認します。

                cv2.rectangle(frame, (x, y), (x+w, y+h), (0, 255, 0), 2)
                # 顔を検出した箇所を四角で囲んでいます。

                id_count, confidence = recognizer.predict(grayimg[y:y+h, x:x+w])
                # 顔判別を行います。
                # id_countは判別した顔のIDNo.です。confidenceは顔判別の信頼度です。

                cv2.putText(frame, str(id_count), (x+5, y-5), font, 1, (255, 255, 255), 2)
                cv2.putText(frame, str(round(confidence, 1)), (x+5, y+h-5), font, 1, (255, 255, 0), 1)
                # 判別した顔のIDNo.と顔判別の信頼度を画面に表示します。

                if confidence >= 100:
                    # 顔判別の信頼度が低い場合は不審者と判断します。
                    id_count = 9

                face_counter[id_count] = face_counter[id_count]+2
                # 顔判別カウンターをカウントアップします。

                face_position[id_count][0] = x+w/2-SCREEN_WIDTH/2
                face_position[id_count][1] = w
                # 顔の位置（中心が0で右が＋左が-）と顔のサイズ（すなわち近さ）を把握します。

        for i in range(10):
            face_counter[i] = face_counter[i]-1
            # 顔がなくなるとカウントを１ずつ減らしていきます。

            if face_counter[i] > 10:
                face_counter[i] = 10  # 上限10
            if face_counter[i] < 0:
                face_counter[i] = 0  # 下限0
            # カウントは0〜10としています。

        cv2.imshow('camera capture', frame)
        # カメラ画像をウインドウに表示させています。

        cv2.waitKey(100)
        # 処理周期を100msecに設定しています。

    cap.release()
    cv2.destroyAllWindows()
    # サブスレッド終了時、カメラをリリースしウインドウを閉じます。


th_facerecog = threading.Thread(target=siro_facerecog, daemon=True)
th_facerecog.start()
print("face recognition start.....",)
# 定義した関数をサブスレッドで起動します。
# 顔認識処理は常時処理で且つ処理が重いのでメイン処理に影響を与えない様にサブスレッドで処理します。


# 基本機能5 音声認識 Julius ***********************************************

cmd = "bash /home/kuro/kurosSIRO/julius/julius_module.sh"
subprocess.Popen(cmd.split())
print("Julius server start.....")
# Juliusをモジュールモードで起動します。
# subprocess.Popenを使って並列処理の子プロセスとして実行しています。


time.sleep(5)
# Juliusの起動を待ちます。5秒は適当です。

client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
HOST = "localhost"  # JuliusサーバーのIPアドレス,自分自身なのでlocalhost
PORT_julius = 10500  # Juliusサーバーの接続ポート番号
client.connect((HOST, PORT_julius))
print("Conecting Julius server.....",)
# IPアドレスとポート番号を指定してクライアントとしてJuliusサーバーに接続します。

words = [['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', '']]
# 音声認識結果を格納するためのリストです。

bufsize = 1024
# Juliusサーバーからデータを読み込む際の1回あたりのデータサイズ[byte]です。

voice_recog = 0
# 音声認識結果読み込み完了フラグです。
# フラグが立ったらメインスレッドで音声認識結果にリアクションします。


def julius_crient():
    # Juliusサーバーから音声認識結果を読み込む関数です。

    global thread_loop, words, voice_recog, speaking

    while thread_loop:
        # メイン処理終了時にthread_loopを0にしてループから脱出させてサブスレッドを終了させます。

        if voice_recog == 0:
            # 音声認識読み込み完了フラグが下がっていたらJuliusからデータの読み込みを開始します。

            julius_data = ""
            NG = 0
            # まずは初期化します。

            while julius_data.find("</RECOGOUT>") == -1 and julius_data.find("<RECOGFAIL/>") == -1:
                julius_data = julius_data + client.recv(1024).decode('utf-8')
                # Juliusからデータを読み込みます。データの中に音声認識完了を示すキーワード</RECOGOUT>
                # 若しくは音声認識失敗を示すキーワード<RECOGFAIL/>のどちらかが現れたら読み込みを終了します。

                if speaking == 1:  # データ読み込み中に発話中フラグが立ったら音声認識結果をキャンセルします。
                    NG = 1

                if thread_loop == 0:  # thread_loopが0になった場合は、読み込みを中止し、whileループから脱出します。
                    break

            # ここからJuliusサーバーから取得したデータ（XML形式）の中から認識した単語”WORD”とその信頼度”CM”の値を抽出します。
            words = [['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', '']]
            counter = 0
            # まずは初期化します。
            for line in julius_data.split('\n'):  # １行ごとに探す
                index = line.find('WORD="')
                if index != -1:  # "WORD="が見つかったら以下を実施
                    words[counter][0] = line[index + 6:line.find('"', index+6)]  # "WORD"の値を格納
                    index = line.find('CM="')
                    words[counter][1] = line[index + 4:line.find('"', index+4)]  # "CM"の値を格納
                    counter = counter + 1  # カウントアップ

            if NG == 1:
                print("発話中のため音声認識結果をスキップ")
                # 発話中だった場合は音声認識結果をスキップします。
            else:
                voice_recog = 1
                # 音声認識結果読み込みフラグを立ててメインスレッドでの処理を待ちます。

            print(julius_data)
            # Juliusサーバーから取得した生データの中身を確認します。

        time.sleep(0.1)
        # 処理周期を調整しています。

    client.close()
    # サブスレッド終了時、Juliusサーバとの通信を終了します。


th_voicerecog = threading.Thread(target=julius_crient, daemon=True)
th_voicerecog.start()
print("Voice recognition start.....",)
# 定義した関数をサブスレッドで起動します。
# 音声認識処理は音声入力待ちが発生するのでメイン処理に影響を与えない様にサブスレッドで処理します。


# 基本機能6 ロボットの記憶 *************************************************

# パーソナルデータ *****************

personal_data = pandas.read_csv('/home/kuro/kurosSIRO/data/personal.csv', header=0).values.tolist()
# pandasを使ってCSVデータを配列で読み込みます。

name = [""]*len(personal_data)  # 名前　
yobi = [""]*len(personal_data)  # よびかた　
suki = [0]*len(personal_data)  # 好感度
for i in range(len(personal_data)):
    name[i] = personal_data[i][1]
    yobi[i] = personal_data[i][2]
    suki[i] = int(personal_data[i][3])
# プログラム中で使用しやすい様に別の変数名に変換します。
# suki(好感度)は数値パラメータなので数値として扱える様、整数型に変換しています。

# 予定データ（週ごと） ************************

schedule1_data = pandas.read_csv('/home/kuro/kurosSIRO/data/schedule1.csv', header=0).values.tolist()
# 同様にpandasを使ってCSVデータを配列で読み込みます。

# 予定データ（日ごと） ***********************

schedule2_data = pandas.read_csv('/home/kuro/kurosSIRO/data/schedule2.csv', header=0).values.tolist()
# 同様にpandasを使ってCSVデータを配列で読み込みます。

schedule2_data_all = ""
for i in range(len(schedule2_data)):
    schedule2_data_all = schedule2_data_all + str(schedule2_data[i][0]) + "/" + str(schedule2_data[i][1]) + "/" + str(schedule2_data[i][2]) + "\t" + str(schedule2_data[i][3]) + "\n"
# こちらは予定データを一つの文字列にまとめたものです。SIROのWebアプリで使用します。


# 基本機能7 障害物検知 ****************************************************

USS_TRIG = 24  # 超音波発生トリガー
USS_ECHO = 25  # エコー受信
# 超音波センサの接続先とGPIOのポート番号を紐づけています。

GPIO.setup(USS_TRIG, GPIO.OUT)
GPIO.setup(USS_ECHO, GPIO.IN)
# トリガー送信を出力、エコー受信を入力に設定しています。

uss_distance = 0  # 障害物までの距離 cm


def siro_uss():
    # 障害物との距離を計測する関数です。

    global uss_distance

    GPIO.output(USS_TRIG, GPIO.HIGH)
    time.sleep(0.00001)
    GPIO.output(USS_TRIG, GPIO.LOW)
    # トリガー信号（10μ秒）を打って超音波を送信します。

    while GPIO.input(USS_ECHO) == GPIO.LOW:
        uss_start = time.time()
        # 照射エコーを検知しなくなるまで待機します。

    while GPIO.input(USS_ECHO) == GPIO.HIGH:
        uss_end = time.time()
        # 反射エコーを検知するまで待機しています。

    uss_distance = (uss_end - uss_start)*17015
    # 超音波を送信してから障害物に当たって跳ね返った反射波を受信するまでの時間から障害物までの距離を算出しています。　
    # 17015は34029 cm/s (音速) ÷２です。往復距離なので最後に２で割っています。


# 基本機能8 気温・湿度検知　************************************************

BME280_sensor = BME280(t_mode=BME280_OSAMPLE_8, p_mode=BME280_OSAMPLE_8,
                       h_mode=BME280_OSAMPLE_8)
# BME280クラスのインスタンス生成、初期化を行っています。

temp = 0.0  # 温度
humid = 0.0  # 湿度
temp_offset = -2.1
humid_offset = 2.0
WBGT = 0.0  # 暑さ指数
# 家の温湿度計と差があるのでオフセットをつけて調整しています。


def siro_ondo():
    # 気温、湿度を検知、WBGTを算出する関数です。

    global temp, humid, WBGT, temp_offset, humid_offset

    temp = BME280_sensor.read_temperature() + temp_offset
    humid = BME280_sensor.read_humidity() + humid_offset
    # クラスメソッドを使ってセンサから気温と湿度を読み込んでいます。

    WBGT = 0.725*temp + 0.0368*humid + 0.00364*temp*humid - 3.246
    # WBGT（暑さ指数）は熱中症予防を目的とした湿度、日射・輻射など周辺の熱環境、気温の３つを取り入れた指標です。
    # 本来のWBGT計算式は複雑なのですが、ここでは屋内前提の近似式を使っています。


# 基本機能9 ジェスチャー認識 ************************************************

gesture_result = 0
# ジェスチャー認識結果を保存する変数です。
# ジェスチャーを検知すると１～9の数字が返ってきます。ジェスチャー未検知の場合は 0 が返ってきます。
# 	0:nothing
# 	1:Forward
# 	2:Backward
# 	3:Right
# 	4:Left
# 	5:Up
# 	6:Down
# 	7:Clockwise
# 	8:anti-clockwise
# 	9:wave

list_gesture = ['なし', '前', '後', '右', '左', '上', '下', '時計回り', '半時計回り', 'バイバイ']

g = gesture()
g.init()
# gestureクラスのインスタンス生成、初期化を行っています。

# 基本機能10 バッテリー残量検知 *********************************************

version = ""
vin = ""  # UPSへの入力：UPSが充電中であれば“GOOD”,充電していなければ“NG”となります。
vout = 0  # UPS出力電圧[mV]
batcap = 0  # バッテリー残量 0～100[％]

UPS = UPS2("/dev/ttyAMA0")
# UPS2クラスのインスタンス生成、初期化処理を行っています。


def siro_battery():
    # バッテリー残量を検知する関数です。

    global version, vin, vout, batcap

    version, vin, batcapstr, voutstr = UPS.decode_uart()

    vout = int(voutstr)
    batcap = int(batcapstr)
    # 数値に変換します。


# 基本機能11 CPU異常検知　*************************************************

# 温度監視　*****************************************

cpu_temp = 0  # CPU温度[℃]


def siro_cpu_temp():
    # CPU温度[℃]を取得する関数です。

    global cpu_temp

    cmd = "vcgencmd measure_temp"
    # Raspberry PiのCPU温度を調べるコマンドです。

    result = subprocess.check_output(cmd.split())
    # コマンドを実行してその結果を取得しています。

    result = result.decode("utf-8")
    # コマンド実行結果をデコードして文字列に変換しています。

    index = result.find('temp=')
    index2 = result.find("'")
    cpu_temp = float(result[index+5:index2])
    # 文字列 temp=xx.x’℃　から数値の部分xx.xを抽出して最後にfroatで文字列を数値に変換しています。


# クロックダウン監視　**********************************

cpu_lowvolt = 0
cpu_clockdown = 0
# CPU電圧低下、クロックダウン検出フラグです。
# 異常を検出したらフラグを立てます。


def siro_cpu_clockdown():
    # CPU電圧低下・クロックダウンを検知する関数です。

    global cpu_clockdown, cpu_lowvolt

    cmd = "vcgencmd get_throttled"
    # Raspberry PiのCPU電圧低下・クロックダウンの有無を調べるコマンドです。

    result = subprocess.check_output(cmd.split())
    # コマンドを実行してその結果を取得しています。

    result = result.decode("utf-8")
    # コマンドの結果をデコードして文字列に変換しています。

    # コマンド実行結果は以下のようになります。
    # ”throttled=0x■■■■■■■”
    # 0x0	    正常
    # 0x50000   過去に低電圧状態になった。現在は正常
    # 0x50005   現在、低電圧状態
    # 0x80000   過去に熱によりクロックダウンした。現在は正常
    # 0x80008   現在、熱によりクロックダウン中
    # 0xD000D   現在、低電圧状態かつ熱によりクロックダウン中

    if ("0x50005" in result):  # 現在、低電圧状態にある。
        cpu_lowvolt = 1
        cpu_clockdown = 0
    elif ("0x80008" in result):  # 現在、熱によりクロックダウンしている。
        cpu_lowvolt = 0
        cpu_clockdown = 1
    elif ("0xD000D" in result):  # 現在、低電圧状態かつ熱によりクロックダウンしている。
        cpu_lowvolt = 1
        cpu_clockdown = 1
    elif (result == "throttled=0x0\n"):  # 異常なし
        cpu_clockdown = 0
        cpu_lowvolt = 0
    # 電圧低下、クロックダウンを検知したら検出フラグを立てます。


# 基本機能12 IPアドレス確認 ************************************************

localipaddress = ""
globalipaddress = ""
# ローカルIPアドレスとグローバルIPアドレス


def siro_localipaddress():
    # ローカルIPアドレスを取得する関数です。

    global localipaddress

    cmd = "hostname -I"
    # ローカルIPアドレスを取得するコマンドです。

    result = subprocess.check_output(cmd.split())
    # コマンドを実行してその結果を取得しています。

    result = result.decode("utf-8")
    # コマンド実行結果をデコードして文字列に変換しています。

    index = result.find('\n')
    localipaddress = result[0:index-1]
    # 文字列の最後の改行とスペースを除いています。


def siro_globalipaddress():
    # グローバルIPアドレスを取得する関数です。

    global globalipaddress

    cmd = "curl ifconfig.me"
    # グローバルIPアドレスを取得するコマンドです。

    result = subprocess.check_output(cmd.split())
    # コマンドを実行してその結果を取得しています。

    result = result.decode("utf-8")
    # コマンド実行結果をデコードして文字列に変換しています。

    globalipaddress = result


# 基本機能13 写真撮影 *****************************************************

def siro_camera():
    # カメラで写真を撮影する関数です。

    now = datetime.datetime.now()
    # 日付と時刻を取得しています。

    filename = "/home/kuro/kurosSIRO/data/picture/" + now.strftime("%Y-%m-%d_%H-%M-%S") + ".jpg"
    # 日付と時刻からファイル名を作成しています。

    ret, frame = cap.read()  # 画像取得
    cv2.imwrite(filename, frame)  # 画像保存
    # カメラ画像を取得しファイル名を指定して保存しています。


# 基本機能14 音声録音 *****************************************************

def siro_voice_record(sec, filename):
    # 音声録音を行う関数です。
    # 引数secで録音時間(秒)で指定し、引数filenameでファイル名を指定します。

    cmd = "arecord -f cd -d " + str(sec) + " /home/kuro/kurosSIRO/data/message/" + filename + ".wav"
    # arecord という標準コマンドを使って録音を行います。
    # arecorodのオプション：　-f cd :CDと同じ音質　-d 録音時間指定
    # 事前に音声ファイルを保存するフォルダ（筆者の場合は/home/kuro/kurosSIRO/data/message）を準備しておきます。

    subprocess.call(cmd.split())
    # コマンドを実行し録音を開始します。


# 基本機能15 radiko視聴 **************************************************

radiko_ch = 0
radiko_ch_list = ["KISSFMKOBE", "802", "CCL"]
# チャンネル選択用の変数とチャンネルリストです。
# radikoで無料視聴できるのはネット接続したエリア内の放送局です。筆者の場合は兵庫県の3局になります。

radio_vol = 70
# ラジオの音量 50-100 で調整

radio_on = 0
# ラジオのON/OFFを示すフラグです。


def siro_radio(command):
    # radikoの再生、停止を行う関数です。引数commandで操作内容を指定します。

    global radiko_ch, radiko_ch_list, radio_on, radio_proc

    if (command == "start"):
        # 引数commandが”start”なら再生します。

        cmd = "amixer cset numid=3 " + str(radio_vol) + "%"
        subprocess.call(cmd.split())
        # ラジオの音量をセットします。

        cmd = "bash /home/kuro/kurosSIRO/play_radiko.sh -t radiko -s " + radiko_ch_list[radiko_ch]
        # radiko再生用シェルスクリプトを起動するコマンドです。チャンネルを指定してradikoを再生します。

        radio_proc = subprocess.Popen(cmd.split())
        # subprocess.Popenを使って並列処理の子プロセスとしてシェルスクリプトを実行しています。
        # ラジオを停止（子プロセスをKILL）する際にプロセスIDが必要となるので
        # 戻り値のオブジェクトをグローバル変数radio_procに保存しておきます。

        radio_on = 1
        # ラジオONフラグを立てます。
        print("ラジオON", radiko_ch_list[radiko_ch], radiko_ch)

    elif (command == "stop"):
        # 引数commandが”stop”なら停止します。

        cmd = "ps ho pid --ppid=" + str(radio_proc.pid)
        # 親プロセスPID（プロセス番号）を指定してその子プロセスのPIDを表示させるコマンドです。

        proc2 = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE)
        proc2.wait()
        # subprocess.Popenを使ってコマンドを実行しています。

        for line in proc2.stdout:
            cmd = 'kill -s SIGINT ' + line.decode()
            subprocess.call(cmd.split())
            # 子プロセスを1つずつKILLします。

        cmd = 'kill -s SIGKILL ' + str(radio_proc.pid)
        subprocess.call(cmd.split())
        # 子プロセスのKILLが終わったら最後に親プロセス（シェルスクリプト）をKILLします。

        radio_on = 0
        # ラジオONフラグを下げます。
        print("ラジオOFF")


# 基本機能16 LINE発信　LINENotify *****************************************

LINETOKEN = "□□□□□□□□□□□□□□□□□□□□□□"
# LINE通知用トークン ※自分のLINEのマイページから取得します。

url_LINENotifyAPI = "https://notify-api.line.me/api/notify"
# LINENotyfyのAPIのURLです。


def siro_Line(LINE_message):
    # LINENotifyを使って自分のLINEにメッセージを通知する関数です。
    # 引数LINE_messageで通知するメッセージ（文字列）を指定します。

    headers = {'Authorization': 'Bearer ' + LINETOKEN}
    payload = {'message': LINE_message}
    requests.post(url_LINENotifyAPI, headers=headers, data=payload)
    # LINENotifyAPIにPOSTリクエストを送っています。


def siro_Line_file(LINE_message, LINE_file):
    # LINENotifyを使って自分のLINEにメッセージ通知+ファイル送信を行う関数です。
    # 引数LINE_messageで通知する文字列を指定し、引数LINE_fileで送信するファイルを指定します。

    headers = {'Authorization': 'Bearer ' + LINETOKEN}
    payload = {'message': LINE_message}
    files = {"imageFile": open(LINE_file, "rb")}
    requests.post(url_LINENotifyAPI, headers=headers, params=payload, files=files)
    # LINENotifyAPIにPOSTリクエストを送っています。


# 基本機能18 ウィキペディア検索 *********************************************

wiki_result = ""
# ウィキペディア検索結果を保存する変数です。


def wikipedia_search(word):
    # ウィキペディア検索を行う関数です。引数word（文字列）で検索ワードを指定します。

    global wiki_result

    wikipedia.set_lang('ja')
    # 言語に日本語を設定しています。

    candidate_list = wikipedia.search(word)
    # 引数wordをウィキペディア検索しています。

    if not candidate_list:

        wiki_result = "該当する単語が存在しませんでした。"
        # 該当する単語がなければ該当なしと回答します。

    else:
        try:
            wiki_result = wikipedia.summary(candidate_list[0])
            # 要約部分を抽出しています。

        except Exception as e:
            # エラーの場合もプログラムが停止しないよう例外処理を行います。
            wiki_result = "検索エラーです。混乱してます。"


# 基本機能19 Webアプリ flask **********************************************

id_pwd = {"****": "********"}
# WebアプリのログインＩＤとパスワードを設定しています。（＊＊＊＊は任意に設定可能です。）

app = Flask(__name__, static_folder='/home/kuro/kurosSIRO/data', template_folder='/home/kuro/kurosSIRO/templates')
# Flaskクラスのインスタンスを生成しています。
# 同時にHTMLファイル他コンテンツファイルの保存場所を指定しています。
# テンプレートファイルの保存場所はデフォルトですが、一応、明記しています。

app.config['TEMPLATES_AUTO_RELOAD'] = True
app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0
# HTMLファイル、画像ファイルなどコンテンツファイルの読み込み時にキャッシュを使用しない様に設定しています。

key = os.urandom(21)
app.secret_key = key
# ログイン情報のセッション管理に使用する秘密鍵を生成しています。


# 以降はルーティング設定です。Webブラウザからのリクエストに対してWebサーバーがどのような処理を行うかを設定します。

# ルート ************************************
@app.route('/')
# Webブラウザがルート（/）をリクエストした場合の処理です。
def login_check():

    if not session.get('login'):
        return redirect('/login')
        # loginセッションがFalseすなわちログイン前であればログイン画面（/login）に移行します。
    else:
        return redirect('/menu')
        # loginセッションがTrueすなわちログイン済であればメニュー画面（/menu）に移行します。


# ログイン画面 *******************************
@app.route('/login')
# Webブラウザがログイン画面（/login）をリクエストした場合の処理です。
def login():

    return render_template("login.html")
    # Flaskのレンダリングテンプレートという機能を使ってログイン画面を表示します。
    # HTMLファイルは事前に所定のフォルダ（筆者の場合は/home/kuro/kurosSIRO/templates）に保存しておきます。


# ログインチェック ****************************
@app.route('/logincheck', methods=['POST'])
# WebブラウザがPOSTメソッドで/logincheckをリクエストした場合の処理です。
# 上記リクエストはログイン画面でログインボタンを押した場合に発生します。
# その動作はHTMLファイル（login.html）側での設定になります。
def logincheck():

    user_id = request.form['user_id']
    password = request.form['password']
    # POSTされたフォーム入力データ（ユーザーIDとパスワード）を取得しています。

    if user_id in id_pwd:
        if password == id_pwd[user_id]:
            session['login'] = True
        else:
            session['login'] = False
    else:
        session['login'] = False
    # ユーザーID、パスワードともに一致すればloginセッションをTrueにします。

    if not session.get('login'):
        return render_template("login.html", warning="ユーザーID,パスワードが一致しません。")
        # loginセッションがFalseなら"ユーザーID,パスワードが一致しません。"と警告します。
        # Flaskのレンダリングテンプレート機能でlogin.htmlの中の｛｛warning｝｝という変数に警告文を反映させます。
    else:
        return redirect('/menu')
        # loginセッションがTrueならメニュー画面(/menu)に移行します。


# メニュー画面 *************************
@app.route('/menu')
# Webブラウザがメニュー画面（/menu）をリクエストした場合の処理です。
def menu():

    if not session.get('login'):
        return redirect('/login')
        # loginセッションがFalseすなわちログイン前ならログイン画面に移行します。
    else:
        return render_template("menu.html")
        # loginセッションがTrueすなわちログイン済であれば
        # Flaskのレンダリングテンプレート機能を使ってメニュー画面を表示します。


# ログアウト処理 **************************
@app.route('/logout')
# Webブラウザが/logoutをリクエストした場合の処理です。
# 上記リクエストはメニュー画面でlogoutボタンを押したとき場合に発生します。
# その動作はHTMLファイル（menu.html）側での設定になります。
def logout():

    session.pop('login', None)
    # loginセッションを削除します。

    return redirect('/login')
    # ログイン画面に移行します。


# ルーティング設定が終わったらWebサーバーを起動します。
def httpserver():
    # Webサーバーの起動を行う関数です。

    app.run(host='0.0.0.0', port=5000)
    # Webサーバーを起動しています。
    # host=’0,0,0,0’と指定することでローカルネットワークの外からもWebサーバーにアクセスすることが可能になります。
    # 0.0.0.0はすべてのホストからのアクセスを受け付けることを意味します。
    # デフォルト設定では自分自身からしかアクセスできないようになっています。
    # ポート番号を5000に指定しています。デフォルトで5000なのですが、一応、明示しています。


th_flask = threading.Thread(target=httpserver, daemon=True)
th_flask.start()
print("SIRO Web application by Flask start...")
# 定義した関数をサブスレッドで起動します。
# Webサーバーの起動はメイン処理と並列処理となるようサブスレッドで処理します。


# 応用機能1,2 障害物監視、タッチセンサ　**************************************

uss_alert = 0
# 障害物アラートフラグです。障害物との距離が10cmを下回るとフラグを立てます。

uss_touch = 0
# タッチ検出フラグです。障害物との距離が5cm以下になるとタッチと判断しフラグを立てます。


def siro_usskanshi():
    # 障害物監視、タッチ検出を行う関数です。
    # 障害物との距離が10cmを下回った場合、障害物アラートフラグを立てます。
    # SIROが移動中の場合は一旦モータを停止させます。
    # また障害物との距離が5cm以下の場合、タッチ検出フラグを立てます。

    global uss_distance, uss_alert, uss_touch, thread_loop

    while thread_loop:
        # メイン処理終了時にthread_loopを0にしてループから脱出させてサブスレッドを終了させます。

        siro_uss()
        # まずは障害物までの距離を計測します。

        if uss_distance < 10:
            # 10cmを下回ると障害物アラートフラグを立てます。
            if uss_alert == 0:
                siro_motor("stop")
                # 障害物アラート検出直後はモータを一度停止させます。
            uss_alert = 1
        else:
            uss_alert = 0

        if uss_distance <= 5:
            # 5cm以下でタッチ検出フラグを立てます。
            uss_touch = 1
        else:
            uss_touch = 0

        time.sleep(0.5)
        # 0.5秒ごとに監視を行います。


th_uss = threading.Thread(target=siro_usskanshi, daemon=True)
th_uss.start()
print("Obstacle detection start.....")
# 定義した関数をサブスレッドで起動します。
# 障害物監視はリアルタイム性が求められる為、処理の遅れが発生しない様、サブスレッドで処理します。


# 応用機能3 気温・湿度監視 *************************************************

temp_kanshi_flag = 0
# 一定時間（10分）ごとに気温・湿度の監視を行うための実行フラグです。
# 監視を行ったらフラグを立て一定時間(10分)待ちます。

tempkanshitime = 0
# 監視を行った時刻を記録するための変数です。


def siro_ondokanshi():
    # 気温・湿度・WBGTの監視を行う関数です。
    # 気温・湿度を検知し気温またはWBGTが高いと”暑くないですか？”などと声掛けをします。

    global temp_kanshi_flag, tempkanshitime

    now_UNIX = time.time()
    # 現在の時刻（UNIX秒）を取得します。

    if temp_kanshi_flag == 0:
        # 実行フラグが下がっていたら監視を行います。

        siro_ondo()
        # 気温と湿度を取得します。

        if WBGT < 30 and WBGT > 25:
            serif = "現在、気温{0:0.1f}度、湿度{1:0.1f}パーセント、WBGTがすこし高いです。エアコン調整しませんか？".format(temp, humid)
            siro_speak(serif)
            # 気温とWBGTが高いと声掛けをします。
        elif WBGT >= 30:
            serif = "現在、気温{0:0.1f}度、湿度は{1:0.1f}パーセント、WBGTが高いです。エアコンきいてますか？".format(temp, humid)
            siro_speak(serif)
        elif temp < 30 and temp > 28:
            serif = "現在、気温{0:0.1f}度、湿度は{1:0.1f}パーセント、暑くないですか？".format(temp, humid)
            siro_speak(serif)
        elif temp >= 30:
            serif = "現在、気温{0:0.1f}度、湿度は{1:0.1f}パーセント、暑いです。".format(temp, humid)
            siro_speak(serif)
        # もっと増やしていく予定ですが今のところはここまでです。

        temp_kanshi_flag = 1
        tempkanshitime = now_UNIX
        # 実行フラグを立てて監視時刻を記録します。

    if now_UNIX - tempkanshitime > 600:
        # 前回の監視から10分経過したら実行フラグを下げます。
        temp_kanshi_flag = 0


# 応用機能4 バッテリー残量監視 **********************************************

battery_kanshi_flag = 0
# 一定時間（5分）ごとにバッテリーの監視を行うための実行フラグです。
# 監視を行ったらフラグを立て一定時間(5分)待ちます。

batterykanshitime = 0
# 監視を行った時刻を記録するための変数です。


def siro_batterykanshi():
    # バッテリー残量の監視を行う関数です。
    # バッテリー残量を検知し30％を下回るとバッテリー低下を知らせます。

    global batcap, battery_kanshi_flag, batterykanshitime

    now_UNIX = time.time()
    # 現在の時刻（UNIX秒）を取得します。

    if battery_kanshi_flag == 0:
        # 実行フラグが下がっていたら監視を行います。

        siro_battery()
        # バッテリー残量を取得します。

        if batcap < 30:
            # バッテリー残量が30％を下回ると警告します。
            serif = "バッテリーが{0:0.1f}パーセントです。バッテリが低下しています。".format(batcap)
            siro_speak(serif)

        battery_kanshi_flag = 1
        batterykanshitime = now_UNIX
        # 実行フラグを立てて監視時刻を記録します。

    if now_UNIX - batterykanshitime > 300:
        # 前回の監視から5分経過したら実行フラグを下げます。
        battery_kanshi_flag = 0


# 応用機能5 CPU異常監視 ***************************************************

cpu_temp_kanshi_flag = 0
cputempkanshitime = 0
# 一定時間（5分）ごとにCPU温度監視を行うための実行フラグです。
# 監視を行ったらフラグを立て一定時間(5分)待ちます。

clockdown_kanshi_flag = 0
clockdownkanshitime = 0
# 一定時間（1分）ごとに電圧低下・クロックダウン監視を行うための実行フラグです。
# 監視を行ったらフラグを立て一定時間(1分)待ちます。


def siro_cpu_temp_kanshi():
    # CPUの異常監視を行う関数です。

    global cpu_temp, cpu_temp_kanshi_flag, cputempkanshitime

    now_UNIX = time.time()
    # 現在の時刻（UNIX秒）を取得します。

    if cpu_temp_kanshi_flag == 0:
        # 実行フラグが下がっていたら監視を行います。

        siro_cpu_temp()
        # CPU温度を取得します。

        if cpu_temp > 50:
            # CPU温度が50℃を上回ると警告します。

            serif = "CPU温度、{0:0.1f}度です。CPU温度が上昇しています。".format(cpu_temp)
            siro_speak(serif)

        cpu_temp_kanshi_flag = 1
        cputempkanshitime = now_UNIX
        # 実行フラグを立てて監視時刻を記録します。

    if now_UNIX - cputempkanshitime > 300:
        # 前回の監視から5分経過したら実行フラグを下げます。
        cpu_temp_kanshi_flag = 0


def siro_cpu_clockdown_kanshi():

    global cpu_lowvolt, cpu_clockdown, clockdown_kanshi_flag, clockdownkanshitime
    # 電圧低下・クロックダウンの監視を行う関数です。

    siro_cpu_clockdown()
    # 電圧低下・クロックダウン発生有無を確認します。

    now_UNIX = time.time()
    # 現在の時刻（UNIX秒）を取得します。

    if clockdown_kanshi_flag == 0:
        # 実行フラグが下がっていたら監視を行います。

        if cpu_lowvolt == 1 and cpu_clockdown == 0:
            serif = "低電圧状態です。"
            siro_speak(serif)

            clockdown_kanshi_flag = 1
            clockdownkanshitime = now_UNIX
            # 実行フラグを立てて監視時刻を記録します。

        elif cpu_clockdown == 0 and cpu_clockdown == 1:
            serif = "クロックダウン中です。"
            siro_speak(serif)

            clockdown_kanshi_flag = 1
            clockdownkanshitime = now_UNIX
            # 実行フラグを立てて監視時刻を記録します。

        elif cpu_clockdown == 1 and cpu_clockdown == 1:
            serif = "低電圧状態です。さらにクロックダウン中です。"
            siro_speak(serif)

            clockdown_kanshi_flag = 1
            clockdownkanshitime = now_UNIX
            # 実行フラグを立てて監視時刻を記録します。

    if now_UNIX - clockdownkanshitime > 60:
        # 前回の監視から1分経過したら実行フラグを下げます。
        clockdown_kanshi_flag = 0


# 応用機能6 IPアドレスLINE通知 *********************************************
def siro_ipaddressLINE():
    # Webアプリで使用するIPアドレスをLINE通知する関数です。

    global localipaddress, globalipaddress

    siro_localipaddress()
    # ローカルIPアドレスを取得します。

    LINE_message = "http://" + localipaddress+":5000"
    siro_Line(LINE_message)
    # LINEで通知します。

    siro_globalipaddress()
    # グローバルIPアドレスを取得します。

    LINE_message = "http://" + globalipaddress+":5000"
    siro_Line(LINE_message)
    # LINEで通知します。


# 応用機能7 写真撮影LINE通知 ***********************************************

def siro_camera_Line():
    # カメラで写真を撮ってLINE通知する関数です。

    now = datetime.datetime.now()
    # 日付と時刻を取得します。

    filename = "/home/kuro/kurosSIRO/data/picture/" + now.strftime("%Y-%m-%d_%H-%M-%S")+".jpg"
    # 日付と時刻からファイル名を作成します。

    ret, frame = cap.read()  # カメラ画像取得
    cv2.imwrite(filename, frame)  # 画像保存
    # カメラ画像を取得しファイル名を指定して保存しています。

    siro_Line_file(filename, filename)
    # 画像をLINE通知します。


# 応用機能8 あいさつ ******************************************************

def siro_greeting(who):
    # SIROにあいさつをさせる関数です。
    # 引数whoであいさつする人をIDNo.で指定します。指定しない場合は100を入力します。

    global yobi

    now = datetime.datetime.now()
    # 時刻を取得します。

    if who == 100:
        # あいさつをする人を指定しない場合は100

        if now.hour >= 0 and now.hour < 9:
            siro_speak("おはようございます。")
        elif now.hour >= 9 and now.hour < 18:
            siro_speak("こんにちわです。")
        elif now.hour >= 18 and now.hour < 21:
            siro_speak("こんばんわです。")
        elif now.hour >= 21 and now.hour < 24:
            siro_speak("ねむたいです。")
        # 時刻によって適切なあいさつをします。

    else:
        # あいさつをする人をIDNo.で指定します。
        if now.hour >= 0 and now.hour < 9:
            siro_speak(yobi[who] + "、おはようございます。")
        elif now.hour >= 9 and now.hour < 18:
            siro_speak(yobi[who] + "、こんにちわです。")
        elif now.hour >= 18 and now.hour < 21:
            siro_speak(yobi[who] + "、こんばんわです。")
        elif now.hour >= 21 and now.hour < 24:
            siro_speak("ねむたいです。")


# 応用機能9 予定を伝える(曜日ごと) ******************************************

def siro_schedule1(who, when):
    # 予定データ(曜日)から予定を抽出する関数です。
    # 引数whoで誰の予定かを選択し、引数whenでいつの予定を確認するかを選択します。

    global schedule1_data

    now = datetime.datetime.now()
    # 日付と時刻を取得します。

    yotei = ""

    if who == 100:
        # 人を選択しない場合は100を入力します。

        if when == "今日":
            for i in range(len(schedule1_data)):
                if schedule1_data[i][now.weekday()+3] != "x":
                    yotei = yotei + schedule1_data[i][2] + 'の' + schedule1_data[i][now.weekday() + 3] + "、"
            if yotei == "":
                yotei2 = "今日は予定はありません。"
            else:
                yotei2 = "今日の予定は" + yotei + "以上です。"
        if when == "明日":
            for i in range(len(schedule1_data)):
                if now.weekday() == 6:
                    if schedule1_data[i][3] != "x":
                        yotei = yotei + schedule1_data[i][2] + "の" + schedule1_data[i][3] + "、"
                else:
                    if schedule1_data[i][now.weekday() + 4] != "x":
                        yotei = yotei + schedule1_data[i][2] + "の" + schedule1_data[i][now.weekday() + 4] + "、"
            if yotei == "":
                yotei2 = "明日は予定はありません。"
            else:
                yotei2 = "明日の予定は" + yotei + "以上です。"
        if when == "明後日":
            for i in range(len(schedule1_data)):
                if now.weekday() == 5:
                    if schedule1_data[i][3] != "x":
                        yotei = yotei + schedule1_data[i][2] + "の" + schedule1_data[i][3] + "、"
                if now.weekday() == 6:
                    if schedule1_data[i][4] != "x":
                        yotei = yotei + schedule1_data[i][2] + "の" + schedule1_data[i][4] + "、"
                else:
                    if schedule1_data[i][now.weekday()+5] != "x":
                        yotei = yotei + schedule1_data[i][2] + "の" + schedule1_data[i][now.weekday() + 5] + "、"
            if yotei == "":
                yotei2 = "明後日は予定はありません。"
            else:
                yotei2 = "明後日の予定は" + yotei + "以上です。"

    else:
        # 人を選択する場合
        if when == "今日":
            if schedule1_data[who][now.weekday()+3] != "x":
                yotei = yotei + schedule1_data[who][2] + 'の' + schedule1_data[who][now.weekday() + 3] + "、"
            if yotei == "":
                yotei2 = "今日は予定はありません。"
            else:
                yotei2 = "今日の予定は" + yotei + "です。"
        if when == "明日":
            if now.weekday() == 6:
                if schedule1_data[who][3] != "x":
                    yotei = yotei + schedule1_data[who][2] + "の" + schedule1_data[who][3] + "、"
            else:
                if schedule1_data[who][now.weekday() + 4] != "x":
                    yotei = yotei + schedule1_data[who][2] + "の" + schedule1_data[who][now.weekday() + 4] + "、"
            if yotei == "":
                yotei2 = "明日は予定はありません。"
            else:
                yotei2 = "明日の予定は" + yotei + "です。"
        if when == "明後日":
            if now.weekday() == 5:
                if schedule1_data[who][3] != "x":
                    yotei = yotei + schedule1_data[who][2] + "の" + schedule1_data[who][3] + "、"
            if now.weekday() == 6:
                if schedule1_data[who][4] != "x":
                    yotei = yotei + schedule1_data[who][2] + "の" + schedule1_data[who][4] + "、"
            else:
                if schedule1_data[who][now.weekday()+5] != "x":
                    yotei = yotei + schedule1_data[who][2] + "の" + schedule1_data[who][now.weekday() + 5] + "、"
            if yotei == "":
                yotei2 = "明後日は予定はありません。"
            else:
                yotei2 = "明後日の予定は" + yotei + "です。"

    return yotei2


# 応用機能10 予定を伝える（日ごと）*******************************************

def siro_schedule2(when):
    # 予定データ(日)から予定を抽出する関数です。
    # 引数whenでいつの予定を確認するかを選択します。

    global schedule2_data

    now = datetime.datetime.now()
    # 日付と時刻を取得します。

    yotei = ""

    if when == "今日":
        for i in range(len(schedule2_data)):
            if schedule2_data[i][0] == now.year:
                if schedule2_data[i][2] == now.day:
                    yotei = yotei + schedule2_data[i][3] + "、"

        if yotei == "":
            yotei2 = "今日は予定はありません。"
        else:
            yotei2 = "今日は" + yotei + "です。"

    elif when == "今月":
        for i in range(len(schedule2_data)):
            if schedule2_data[i][0] == now.year:
                if schedule2_data[i][1] == now.month:
                    if schedule2_data[i][2] >= now.day:
                        yotei = yotei + str(schedule2_data[i][1]) + "月" + str(schedule2_data[i][2]) + "日、" + str(schedule2_data[i][3]) + "、"
        if (yotei == ""):
            yotei2 = "今月の予定はありません。"
        else:
            yotei2 = "今月の予定は" + yotei + "です。"

    elif when == "今年":
        for i in range(len(schedule2_data)):
            if schedule2_data[i][0] >= now.year:
                if schedule2_data[i][1] >= now.month:
                    if schedule2_data[i][2] >= now.day:
                        yotei = yotei + str(schedule2_data[i][1]) + "月" + str(schedule2_data[i][2]) + "日、" + str(schedule2_data[i][3]) + "、"
        if yotei == "":
            yotei2 = "今年の予定はありません。"
        else:
            yotei2 = "今年の予定は" + yotei + "です。"

    elif when == "来月":
        if now.month == 12:
            month = 1
            year = now.year + 1
        else:
            month = now.month + 1
            year = now.year
        for i in range(len(schedule2_data)):
            if schedule2_data[i][0] == year:
                if schedule2_data[i][1] == month:
                    yotei = yotei + str(schedule2_data[i][1]) + "月" + str(schedule2_data[i][2]) + "日" + str(schedule2_data[i][3]) + "、"
        if yotei == "":
            yotei2 = "来月の予定はありません。"
        else:
            yotei2 = "来月の予定は" + yotei + "です。"

    elif when == "明日":
        now = now + datetime.timedelta(days=1)
        for i in range(len(schedule2_data)):
            if schedule2_data[i][0] == now.year:
                if schedule2_data[i][1] == now.month:
                    if schedule2_data[i][2] == now.day:
                        yotei = yotei + schedule2_data[i][3] + "、"

        if yotei == "":
            yotei2 = "明日の予定はありません。"
        else:
            yotei2 = "明日は" + yotei + "です。"

    elif when == "明後日":
        now = now + datetime.timedelta(days=2)
        for i in range(len(schedule2_data)):
            if schedule2_data[i][0] == now.year:
                if schedule2_data[i][1] == now.month:
                    if schedule2_data[i][2] == now.day:
                        yotei = yotei + schedule2_data[i][3] + "、"

        if yotei == "":
            yotei2 = "明後日の予定はありません。"
        else:
            yotei2 = "明後日は" + yotei + "です。"

    return yotei2


# 応用機能11 鳩時計 *******************************************************

hatoclock_flag1 = 0
hatoclock_flag2 = 0
# 鳩時計行動フラグです。
# 0分になったら鳩時計（0分）を鳴らしてflag1を立てます。0分以外になったらフラグをおろして次に備えます。
# 30分になったら鳩時計（30分）を鳴らしてflag2を立てます。30分以外になったらフラグをおろして次に備えます。

hatoclocktime = 0
# 鳩時計を鳴らした時刻を記録するための変数です。


def siro_hatoclock():
    # 鳩時計として時刻を知らせる関数です。

    global hatoclock_flag1, hatoclock_flag2, hatoclocktime

    now = datetime.datetime.now()  # datetimeオブジェクト
    now_UNIX = time.time()  # UNIX秒
    # 時刻を取得します。

    if now.minute == 0 and hatoclock_flag1 == 0:
        # 0分になったら時刻の回数ポッポッポッと鳴きます。

        if now.hour > 12:
            for i in range(now.hour - 12):
                siro_sound("はと")
            serif = str(now.hour - 12) + "時です。"
            siro_speak(serif)
        else:
            for i in range(now.hour):
                siro_sound("はと")
            serif = str(now.hour) + "時です。"
            siro_speak(serif)

        hatoclock_flag1 = 1
        hatoclocktime = now_UNIX
        # 何度も鳴かないようにフラグを立てて時刻(UNIX秒)を記録します。

    if now.minute == 30 and hatoclock_flag2 == 0:
        # 30分になったら1回だけ鳴きます。

        siro_sound("はと")
        if now.hour > 12:
            serif = str(now.hour - 12) + "時半です。"
        else:
            serif = str(now.hour) + "時半です。"
        siro_speak(serif)

        hatoclock_flag2 = 1
        hatoclocktime = now_UNIX
        # 何度も鳴かないようにフラグを立てて時刻(UNIX秒)を記録します。

    # 1分経過したらフラグを下げます。
    if hatoclock_flag1 == 1 and now_UNIX - hatoclocktime > 60:
        hatoclock_flag1 = 0
    if hatoclock_flag2 == 1 and now_UNIX - hatoclocktime > 60:
        hatoclock_flag2 = 0


# 応用機能12 メッセージ録音　***********************************************

messages = [[0, 0, datetime.datetime.now()], [0, 0, datetime.datetime.now()], [0, 0, datetime.datetime.now()]]
# メッセージの有無、誰のメッセージか？、録音した日時を保存する変数です。
# メッセージは3件まで録音可能とします。
# まだ１００％完成していませんが、とりあえずメッセージの録音、再生、消去はできるようになりました。


def siro_message(command, kenme):
    # メッセージの録音、再生、削除を行う関数です。
    # 引数commandで録音か再生または削除かを選択します。
    # 削除する場合は引数kenmeで何件目のメッセージを削除するかを指定します。

    global messages

    if command == "録音":

        for i in range(3):
            if messages[i][0] == 0:
                # messageの空きを確認します。

                siro_speak("１５秒でメッセージをどうぞ。")
                siro_sound("ピー")
                cmd = "arecord -f cd -d 15 /home/kuro/kurosSIRO/data/message/{0:d}.wav".format(i)
                subprocess.call(cmd.split())
                siro_speak("録音を終了します。")
                # arecordコマンドで録音します。
                # arecorodのオプション：　-f cd :CDと同じ音質 -d 録音時間

                messages[i][0] = 1
                # 録音済フラグを立てます。

                break
                # メッセージを録音したらforループから脱出します。

            if i == 2:
                # messageに空きがない場合は諦めます。
                siro_speak("既に３件のメッセージが録音されています。")

    elif command == "再生":

        aaa = 0
        for i in range(3):
            aaa = aaa + messages[i][0]
        # メッセージの有無を確認します。

        if aaa == 0:
            # メッセージが無い場合
            siro_speak("メッセージはありません。")

        else:
            # メッセージがある場合
            serif = str(aaa) + "件のメッセージがあります。"
            siro_speak(serif)

            for i in range(3):
                if messages[i][0] == 1:
                    serif = str(i+1) + "件目のメッセージを再生します。"
                    siro_speak(serif)
                    cmd = "aplay /home/kuro/kurosSIRO/data/message/{0:d}.wav".format(i)
                    subprocess.call(cmd.split())
            # 順番にメッセージを再生します。

            serif = "メッセージは以上です。"
            siro_speak(serif)

    elif command == "消去":
        # 引数kenmeで選択したメッセージを消去します。
        # kenmeが10の場合はすべてのメッセージを消去します。

        if kenme == 1:
            siro_speak("一件目のメッセージを消去します。")
            messages[0][0] = 0
            # 録音済フラグを下げます。
            # 音声ファイルは削除せずにそのままです。新たにメッセージを録音する場合はファイルを上書きします。
            siro_sound("メッセージ消去")

        elif kenme == 2:
            siro_speak("二件目のメッセージを消去します。")
            messages[1][0] = 0
            siro_sound("メッセージ消去")
        elif kenme == 3:
            siro_speak("三件目のメッセージを消去します。")
            messages[2][0] = 0
            siro_sound("メッセージ消去")
        elif kenme == 10:
            siro_speak("すべてのメッセージを消去します。")
            for i in range(3):
                messages[i][0] = 0
            siro_sound("メッセージ消去")


# 応用機能13 タイマー　****************************************************

timers = [[0, "", time.time()], [0, "", time.time()], [0, "", time.time()]]
# タイマーセットの有無、セット時間、タイマーセット時刻を保存する変数です。
# タイマーは3件までセット可能とします。


def siro_timer():
    # タイマー処理を行う関数です。

    global thread_loop, timers

    while thread_loop:
        # メイン処理終了時にthread_loopを0にしてループから脱出させてサブスレッドを終了させます。

        now_UNIX = time.time()
        # 時刻（UNIX秒）を確認します。

        for i in range(3):

            if timers[i][0] == 1 and now_UNIX-timers[i][2] > 0:
                # タイマーセットの有無を確認し、タイマーがセットされていればタイマーセット時刻を過ぎていないかを確認します。

                siro_sound("タイマー")
                serif = "{0:d}件目、{1:s}のタイマーを終了します。".format(i+1, timers[i][1])
                siro_speak(serif)
                # タイマーを鳴らします。
                # 本当は”タイマーストップ”というまでタイマーを鳴らし続けたいのですが、SIROはサウンドを鳴らしながらの音声認識ができません。。。
                # Alexa（スマートスピーカー）は本当に優秀です。

                timers[i][0] = 0
                # タイマーをリセットします。

        time.sleep(1)


th_timer = threading.Thread(target=siro_timer, daemon=True)
th_timer.start()
print("Timer start.....")
# 定義した関数をサブスレッドで起動します。
# タイマー処理はリアルタイム性が求められる為、処理の遅れが発生しない様、サブスレッドで処理します。


def siro_timer_set(command, minutes, kenme):
    # タイマーのセット、リセットを行う関数です。
    # 引数commandでセットかリセットを選択し、引数minutesでタイマー時間を指定します。
    # タイマーをリセットする場合は引数kenmeで何件目のタイマーをリセットするかを指定します。

    global timers

    now_UNIX = time.time()
    # 時刻（UNIX秒）を確認します。

    if command == "セット":

        for i in range(3):
            if timers[i][0] == 0:
                # timerの空きを確認します。

                serif = minutes + "のタイマーをセットします。"
                siro_speak(serif)

                timers[i][0] = 1
                # タイマーフラグを立てます。

                timers[i][1] = minutes
                # セット時間（分）をタイマーに入力します。

                index = minutes.find('分')
                minutes2 = int(minutes[0:index])
                # ”○分”から数値部分○を抽出します。

                timers[i][2] = now_UNIX + minutes2*60
                # セット時刻を入力します。

                break
                # タイマーをセットしたらforループから抜けます。

            if i == 2:
                siro_speak("既に３件のタイマーがセットされています。")
                # タイマーがすでに3件セット済であればその旨を伝えます。

    elif command == "確認":
        # タイマーの有無を確認します。

        aaa = 0
        for i in range(3):
            aaa = aaa + timers[i][0]

        if aaa == 0:
            siro_speak("タイマーはセットされていません。")
            # タイマーがセットされていなければその旨を伝えます。

        else:
            serif = str(aaa) + "件のタイマーがあります。"
            siro_speak(serif)

            for i in range(3):
                if timers[i][0] == 1:  # タイマーの空きを確認
                    serif = str(i+1) + "件目"+timers[i][1] + "のタイマーがあります。のこり" + str(int(timers[i][2] - now_UNIX)) + "秒です。"
                    siro_speak(serif)
        # タイマーがセットされていれば、何分のタイマーがのこり何秒かを伝えます。

    elif command == "キャンセル":

        if kenme == 1:
            # 引数kenmeで何件目のタイマーをキャンセルするか選択肢します。
            siro_speak("一件目のタイマーをキャンセルします。")
            siro_sound("メッセージ消去")
            timers[0][0] = 0
            # タイマーフラグを下げます。

        elif kenme == 2:
            siro_speak("二件目のタイマーをキャンセルします。")
            siro_sound("メッセージ消去")
            messages[1][0] = 0

        elif kenme == 3:
            siro_speak("三件目のタイマーをキャンセルします。")
            siro_sound("メッセージ消去")
            messages[2][0] = 0

        elif kenme == 10:
            # 全件キャンセルする場合はkenme=10とします。
            siro_speak("すべてのタイマーをキャンセルします。")
            siro_sound("メッセージ消去")
            for i in range(3):
                timers[i][0] = 0


# 応用機能14 Webアプリ Auto mode ******************************************

@app.route('/automode', methods=['GET'])
# Automode画面です。
# SIROの制御プログラムの動作チェックに使用しています。
# 顔認識処理の状態、音声認識結果、各種センサの値,CPU温度他各種パラメータなどを表示します。
def automode():

    return render_template("automode.html", mode=mode_string[mode], siro_dontmove=siro_dontmove, siro_silent=siro_silent,
                           communication=communication, speaking=speaking, face_counter=face_counter, face_position=face_position, words=words, uss_distance=int(uss_distance), uss_alert=uss_alert,
                           uss_touch=uss_touch, temp=int(temp), humid=int(humid), WBGT=int(WBGT), batcap=batcap, gesture_result=list_gesture[gesture_result],
                           cpu_temp=cpu_temp, cpu_lowvolt=cpu_lowvolt, cpu_clockdown=cpu_clockdown, globalipaddress=globalipaddress, localipaddress=localipaddress,
                           messages=messages, timers=timers)
    # Flaskのレンダリングテンプレートという機能を使ってAutomode画面を表示します。


# 応用機能14 Webアプリ RadioControl **************************************

@app.route('/radiocontrol', methods=['GET'])
# RadioControl画面です。
# SIROをラジコンで操作できます。
# カメラ画像とコントローラーを表示します。
def radiocontrol():

    global mode, g_parameter

    mode = 4  # ラジコンモード
    g_parameter = [0, 0, 0, 0, 0]  # 汎用パラメータ初期化
    # ラジコンモードに移行します。

    return render_template("radiocontrol.html")
    # Flaskのレンダリングテンプレートという機能を使ってRadioControl画面を表示します。


@app.route('/motor_go', methods=['GET'])
# コントローラーの前進ボタンがクリックされたときの処理です。
def motor_go():

    siro_motor("go")
    # 前進します。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


@app.route('/motor_left', methods=['GET'])
# コントローラーの左回転ボタンがクリックされたときの処理です。
def motor_left():

    siro_motor("left")
    # 左回転します。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


@app.route('/motor_right', methods=['GET'])
# コントローラーの右回転ボタンがクリックされたときの処理です。
def motor_right():

    siro_motor("right")
    # 右回転します。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


@app.route('/motor_back', methods=['GET'])
# コントローラーの後進ボタンがクリックされたときの処理です。
def motor_back():

    siro_motor("back")
    # 後進します。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


@app.route('/motor_stop', methods=['GET'])
# コントローラーの停止ボタンがクリックされたときの処理です。
def motor_stop():

    siro_motor("stop")
    # 停止します。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


@app.route('/texttospeak', methods=['POST'])
# コントローラーの発話ボタンがクリックされたときの処理です。
def texttospeak():

    text = request.form['text']
    # 発話する文字列を取得します。

    siro_speak(text)
    # 発話します。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


# カメラ画像表示 **********************
# MotionJPEGという仕組みを使用しています。
# サーバーからJPEG画像を連続して送ることでWebブラウザ上で動画になります。
# Pythonではジェネレータ（yield文）を使って実現します。
# HTMLファイルに <img src="/video_feed"…　と記述すれば表示できます。


@app.route('/video_feed')
def video_feed():
    return Response(gen(),
                    mimetype='multipart/x-mixed-replace; boundary=frame')


def gen():
    # ジェネレータ（Python）

    while True:
        ret, image = cap.read()
        # カメラ画像を取得しています。
        ret, jpeg = cv2.imencode('.jpg', image)
        jpegbytes = jpeg.tobytes()
        # JPEGにしてそれを更にバイト列に変換しています。

        yield (b'--frame\r\n'
               b'Content-Type: image/jpeg\r\n\r\n' + jpegbytes + b'\r\n\r\n')
        # ジェネレータなのでyield文を使います。


# 応用機能14 Webアプリ radiko *********************************************

@app.route('/radiko', methods=['GET'])
# radiko画面です。
# SIROのスピーカーでradikoを聴取できます。
# 放送局のロゴとコントローラー(音量調整と放送局選択)を表示します。
def radiko():

    global mode, g_parameter

    mode = 3  # ラジオモード
    g_parameter = [0, 0, 0, 0, 0]  # 汎用パラメータ初期化
    # ラジオモードに移行します。

    return render_template("radiko.html", radiko_ch=radiko_ch_list[radiko_ch])
    # Flaskのレンダリングテンプレートという機能を使ってradiko画面を表示します。


@app.route('/radio_volup', methods=['GET'])
# コントローラーの▲ボタンがクリックされたときの処理です。
def radio_volup():

    global radio_vol

    radio_vol = radio_vol + 10
    if radio_vol > 100:
        radio_vol = 100
    # 音量パラメータradio_volを上げます。最大100です。

    cmd = "amixer cset numid=3 " + str(radio_vol) + "%"
    subprocess.call(cmd.split())
    # 音量セットします。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


@app.route('/radio_voldown', methods=['GET'])
# コントローラーの▼ボタンがクリックされたときの処理です。
def radio_voldown():

    global radio_vol

    radio_vol = radio_vol - 10
    if radio_vol < 40:
        radio_vol = 40
    # 音量パラメータradio_volを下げます。最小40です。

    cmd = "amixer cset numid=3 " + str(radio_vol) + "%"
    subprocess.call(cmd.split())
    # 音量セットします。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


@app.route('/radio_ch1', methods=['GET'])
# コントローラーの▶▶ボタンがクリックされたときの処理です。
def radio_ch1():

    global radiko_ch

    radiko_ch = radiko_ch + 1
    if radiko_ch == 3:
        radiko_ch = 0
    # 選局パラメータradio_chを+1

    siro_radio("stop")
    siro_radio("start")
    # ラジオを停止、再生することでチャンネル変更しています。

    return render_template("radiko.html", radiko_ch=radiko_ch_list[radiko_ch])
    # 放送局のロゴを変更するため、画面更新します。


@app.route('/radio_ch2', methods=['GET'])
# コントローラーの◀◀ボタンがクリックされたときの処理です。
def radio_ch2():

    global radiko_ch

    radiko_ch = radiko_ch - 1
    if radiko_ch == -1:
        radiko_ch = 2
    # 選局パラメータradio_chを-1

    siro_radio("stop")
    siro_radio("start")
    # ラジオを停止、再生することでチャンネル変更しています。

    return render_template("radiko.html", radiko_ch=radiko_ch_list[radiko_ch])
    # 放送局のロゴを変更するため、画面更新します。


@app.route('/radio_startstop', methods=['GET'])
# コントローラーの▶/■ボタンがクリックされたときの処理です。
def radio_startstop():

    global radio_on

    if radio_on == 1:
        siro_radio("stop")
    else:
        siro_radio("start")
    # ラジオ再生フラグradio_onを確認し再生中なら停止し停止中なら再生します。

    return ('OK', 200)
    # HTTPリスポンスを返します。
    # 画面更新は行いません。


# 応用機能14 Webアプリ Schedule1 *****************************************

ALLOWED_EXTENSIONS = {'txt', 'csv'}
# アップロードを許可するファイル形式を指定しています。


def allwed_file(filename):
    # 許可されたファイルかどうかを確認する関数です。
    # 引数filenameでファイル名を指定します。
    # OKなら１、だめなら0を返します。

    return '.' in filename and filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS


@app.route('/schedule1', methods=['GET'])
# Schedule1画面です。
# 予定データ（週ごと）の確認、編集ができます。
# 予定データ（週ごと）を表示します。また、編集の為のダウンロードボタンとアップロードボタンがあります。
def schedule1():

    return render_template("schedule1.html", schedule1_data=schedule1_data)
    # Flaskのレンダリングテンプレートという機能を使ってSchedule1画面を表示します。


@app.route('/schedule1_download', methods=['GET'])
# ダウンロードボタンがクリックされたときの処理です。
def schedule1_download():

    downloadDir = '/home/kuro/kurosSIRO/data'
    downloadFile = 'schedule1.csv'
    downloadFileName = 'schedule1.csv'

    return send_from_directory(downloadDir, downloadFile, as_attachment=True,
                               attachment_filename=downloadFileName, cache_timeout=0, mimetype="text/csv")
    # 予定データ（週ごと）ファイルを送ります。
    # cache_timeout=0 でキャッシュを無効にしています。(キャッシュを無効にしないと最新ファイルがダウンロードされません。)


@app.route('/schedule1_upload', methods=['POST'])
# アップロードボタンがクリックされたときの処理です。
def schedule1_upload():

    global schedule1_data

    uploadDir = '/home/kuro/kurosSIRO/data'
    uploadFile = 'schedule1.csv'

    if 'file' not in request.files:
        return render_template("schedule1.html", schedule1_data=schedule1_data)
        # ファイルが選択されていない場合は何もせずに画面を再読み込みします。
    file = request.files['file']

    if allwed_file(file.filename) == 0:
        # 許可されたファイルでない場合は何もせずに画面を再読み込みします。
        return render_template("schedule1.html", schedule1_data=schedule1_data)
    else:
        file.save(os.path.join(uploadDir, uploadFile))
        # 許可されたファイルであればファイルを保存します。

        schedule1_data = pandas.read_csv('/home/kuro/kurosSIRO/data/schedule1.csv', header=0).values.tolist()
        # # pandasを使ってCSVデータを配列で読み込みます。

        return render_template("schedule1.html", schedule1_data=schedule1_data)
        # 画面を更新します。


# 応用機能14 Webアプリ Schdule2 *******************************************

@app.route('/schedule2', methods=['GET'])
# Schedule2画面です。
# 予定データ（日ごと）の確認、編集ができます。
# 予定データ（日ごと）を表示します。また、編集の為のダウンロードボタンとアップロードボタンがあります。
def schedule2():
    return render_template("schedule2.html", schedule2_data_all=schedule2_data_all)
    # Flaskのレンダリングテンプレートという機能を使ってSchedule2画面を表示します。


@app.route('/schedule2_download', methods=['GET'])
# ダウンロードボタンがクリックされたときの処理です。
def schedule2_download():
    downloadDir = '/home/kuro/kurosSIRO/data'
    downloadFile = 'schedule2.csv'
    downloadFileName = 'schedule2.csv'

    return send_from_directory(downloadDir, downloadFile, as_attachment=True,
                               attachment_filename=downloadFileName, cache_timeout=0, mimetype="text/csv")
    # 予定データ（日ごと）ファイルを送ります。
    # cache_timeout=0 でキャッシュを無効にしています。(キャッシュを無効にしないと最新ファイルがダウンロードされません。)


@app.route('/schedule2_upload', methods=['POST'])
# アップロードボタンがクリックされたときの処理です。
def schedule2_upload():

    global schedule2_data, schedule2_data_all
    uploadDir = '/home/kuro/kurosSIRO/data'
    uploadFile = 'schedule2.csv'

    if 'file' not in request.files:
        return render_template("schedule2.html", schedule2_data_all=schedule2_data_all)
        # ファイルが選択されていない場合は何もせずに画面を再読み込みします。
    file = request.files['file']

    if allwed_file(file.filename) == 0:
        # 許可されたファイルでない場合は何もせずに画面を再読み込みします。
        return render_template("schedule2.html", schedule2_data_all=schedule2_data_all)
    else:
        file.save(os.path.join(uploadDir, uploadFile))
        # 許可されたファイルであればファイルを保存します。

        schedule2_data = pandas.read_csv('/home/kuro/kurosSIRO/data/schedule2.csv', header=0).values.tolist()
        # pandasを使ってCSVデータを配列で読み込みます。

        schedule2_data_all = ""
        for i in range(len(schedule2_data)):
            schedule2_data_all = schedule2_data_all + str(schedule2_data[i][0]) + "年" + str(schedule2_data[i][1])\
                + "月" + str(schedule2_data[i][2]) + "日: " + str(schedule2_data[i][3]) + "\n"

        return render_template("schedule2.html", schedule2_data_all=schedule2_data_all)
        # 画面を更新します。


# 応用機能14  Webアプリ ウィキペディア検索 ***********************************
# メニュー画面で検索ワードを入力して”ウィキペディア検索”ボタンを押したときの処理です。
@app.route('/wikipedia', methods=['POST'])
def wiki():

    word = request.form['word']
    # 検索ワードをを取得します。

    wikipedia_search(word)
    # ウィキペディア検索します。

    siro_speak2(wiki_result)
    # 検索結果をSIROに読み上げさせます。

    return render_template("wikipedia.html", word=word, result=wiki_result)
    # 検索結果を画面表示します。


def siro_speak2(serif):
    # SIROに発話させる関数パート２です。
    # ウィキペディア検索結果読み上げ用に作成しました。
    # googletexttospeechというPythonモジュールを使用しています。
    # こちらは長文のセリフでも使用できます。(逆にOpenJtalkは長文だとエラーになります。）
    # 発話の開始と同時に検索結果画面が表示されるようsubprocess.Popenを使って並列処理で音声再生します。

    global sp_vol

    tts = gTTS(serif, lang="ja")
    tts.save("/tmp/out.mp3")
    # googletexttospeechを使って音声ファイルを作成します。

    cmd = "amixer cset numid=3 "+str(sp_vol)+"%"
    subprocess.call(cmd.split())
    # 音量を設定します。

    cmd = "mplayer -speed 1.2 -af scaletempo /tmp/out.mp3"
    subprocess.Popen(cmd.split())
    # 作成した音声ファイルを再生します。
    # subprocess.Popenを使って並列処理で再生します。


# 2. メイン処理パート *****************************************************
# ここからようやくメイン処理に入ります。

print("SIRO Start......",)
siro_speak("しろ起動します。")

siro_ipaddressLINE()
# 応用機能6 IPアドレスLINE通知

try:  # try,except 構文, ctl +C (キーボード割り込み)で終了処理に移ります。
    while True:  # メイン処理ループです。

        # 共通処理 *******************************************************

        siro_batterykanshi()
        # 応用機能4 バッテリー残量監視

        siro_cpu_temp_kanshi()
        # 応用機能5 CPU温度監視

        siro_cpu_clockdown_kanshi()
        # 応用機能5 CPU異常監視

        # 1. のんびりモード ***********************************************
        if mode == 1:

            print("のんびりモード。")

            # 初期処理 *******************
            if g_parameter[0] == 0:

                siro_motor_speed = 70  # モータ速度を設定します。
                g_parameter[0] = 1

            # 応用機能3 気温・湿度監視 *********
            siro_ondokanshi()

            # 応用機能11 鳩時計 ****************
            siro_hatoclock()

            # 応用機能15 コミュニケーション *****************
            # 家族の顔を見つけるとコミュニケーションします。
            # コミュニケーションの進度を表すパラメータ（communication）で行動を制御します。

            for i in range(4):
                if face_counter[i] >= 8:
                    if communication[i] == 0:
                        siro_greeting(i)  # 応用機能8 あいさつ
                        communication[i] += 1  # コミュニケーション進度＋１
                    elif communication[i] == 1:
                        serif = siro_schedule1(i, "今日")  # 応用機能9 予定を伝える
                        siro_speak(serif)
                        communication[i] += 1
                    # とりあえず骨子だけ完成。。つづきは今後作成予定です。。。

            # 応用機能16 巡回 *****************************
            # 家の中を巡回します。とりあえずランダムウォークです。
            # 本当は自己位置推定,マッピングなどを行いたいのですが、そのためにはまずセンサの増強が必要です。。。
            # とりあえず骨子だけ完成させてつづきは今後作成予定です。。。

            if siro_dontmove != 1:
                ransuint = random.randint(1, 2000)  # 乱数生成
                if ransuint >= 1 and ransuint < 10:
                    siro_motor("go")  # 前進
                    time.sleep(4)
                    siro_motor("stop")
                elif ransuint >= 10 & ransuint < 20:
                    siro_motor("right")  # 右旋回
                    time.sleep(2)
                    siro_motor("stop")
                elif ransuint >= 20 and ransuint < 30:
                    siro_motor("left")   # 左旋回
                    time.sleep(2)
                    siro_motor("stop")
                elif ransuint >= 30 and ransuint < 40:
                    siro_motor("right")   # 右小旋回
                    time.sleep(1)
                    siro_motor("stop")
                elif ransuint >= 40 and ransuint < 50:
                    siro_motor("left")  # 左小旋回
                    time.sleep(1)
                    siro_motor("stop")

            # 応用機能17 顔サーチ **************************
            # 家族の顔を見つけると振り向きます。

            for i in range(4):
                if face_counter[i] >= 8 and face_position[i][0] != 0:

                    if face_position[i][0] < - face_position[i][1]/2:
                        siro_motor("left")
                        time.sleep(0.4)
                        siro_motor("stop")
                    elif face_position[i][0] > face_position[i][1]/2:
                        siro_motor("right")
                        time.sleep(0.4)
                        siro_motor("stop")

            # 応用機能18 ひとりごと **************************
            # ランダムに独り言を言います。
            if siro_silent != 1:
                ransuint = random.randint(1, 1000)  # 乱数生成
                if ransuint == 1:
                    siro_speak("しろちゃんです。")  # 自己紹介
                elif ransuint == 2:
                    siro_ondo()
                    serif = "げんざい、きおん、{0:0.1f}ど。しつど、{1:0.1f}パーセント".format(temp, humid)  # 気温と湿度を伝える。
                    siro_speak(serif)
                elif ransuint == 3:
                    serif = siro_schedule2("今月")  # 今月の予定を伝える。
                    siro_speak(serif)
                # とりあえず骨子だけ完成。。つづきは今後作成予定です。。。

            # 応用機能19 音声認識へのリアクション ****************
            if voice_recog == 1:
                # 音声認識完了フラグが立っていたら、音声認識結果wordsを確認してリアクションします。

                for i in range(10):
                    print('WORD: {0:s}{1:s}\n'.format(words[i][0], words[i][1]))
                    # 音声認識結果を表示して確認します。

                # 名前を呼ばれたときのリアクション
                if 'しろ' in words[1][0] and '[/s]' == words[2][0] and float(words[1][1]) > 0.8:
                    siro_speak("なんですか？")

                # ワードへのリアクション
                for i in range(10):

                    if 'ラジオ' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_ipaddressLINE()
                        # 応用機能6 IPアドレスLINE通知
                        # ラジオ操作にWebアプリを使用するのでIPアドレスをLINE通知します。

                        mode = 3  # ラジオモード
                        g_parameter = [0, 0, 0, 0, 0]  # 汎用パラメータ初期化
                        # ラジオモードに移行します。
                        break

                    elif 'IPアドレス' in words[i][0] and float(words[i][1]) > 0.8:

                        siro_ipaddressLINE()
                        # 応用機能6 IPアドレスLINE通知
                        break

                    elif 'こんにちは' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_greeting(100)
                        # 応用機能8 あいさつ
                        break

                    elif 'こんばんは' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_greeting(100)
                        # 応用機能8 あいさつ
                        break

                    elif '予定' in words[i][0] and float(words[i][1]) > 0.9:
                        # 応用機能9,10 予定を伝える

                        for j in range(len(words)):
                            if '今日' in words[j][0] and float(words[j][1]) > 0.8:

                                serif = siro_schedule1(100, '今日')
                                siro_speak(serif)
                                serif = siro_schedule2('今日')
                                siro_speak(serif)
                                break

                            elif '明日' in words[j][0] and float(words[j][1]) > 0.8:

                                serif = siro_schedule1(100, '明日')
                                siro_speak(serif)
                                serif = siro_schedule2('明日')
                                siro_speak(serif)
                                break

                            if '明後日' in words[j][0] and float(words[j][1]) > 0.8:

                                serif = siro_schedule1(100, '明後日')
                                siro_speak(serif)
                                serif = siro_schedule2('明後日')
                                siro_speak(serif)
                                break

                            if '今月' in words[j][0] and float(words[j][1]) > 0.8:

                                serif = siro_schedule2("今月")
                                siro_speak(serif)
                                break

                            if '来月' in words[j][0] and float(words[j][1]) > 0.8:

                                serif = siro_schedule2("来月")
                                siro_speak(serif)
                                break

                            if '今年' in words[j][0] and float(words[j][1]) > 0.8:

                                serif = siro_schedule2("今年")
                                siro_speak(serif)
                                break

                    elif 'しゃしん' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_speak("写真をとります。")
                        siro_camera_Line()
                        # 応用機能7 写真撮影LINE発信
                        siro_sound("カメラ")  # シャッター音
                        break

                    elif 'メッセージ' in words[i][0] and float(words[i][1]) > 0.9:
                        # 応用機能12 メッセージ

                        for j in range(len(words)):
                            if '録音' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_message("録音", 0)
                                break
                            elif '再生' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_message("再生", 0)
                                break
                            elif '消去' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("全部ですか？それとも何件目ですか？")
                                mode = 2  # 聞き耳モードに以降
                                g_parameter = [0, 1, 0, 0, 0]  # 聞き耳を立てる対象を汎用パラメータで指定します。
                                break

                    elif 'タイマー' in words[i][0] and float(words[i][1]) > 0.9:
                        # 応用機能13 タイマー

                        for j in range(len(words)):
                            if 'かけて' in words[j][0] and float(words[j][1]) > 0.6:
                                for k in range(len(words)):
                                    if '分' in words[k][0] and float(words[k][1]) > 0.4:
                                        siro_timer_set("セット", words[k][0], 0)
                                        break
                            elif ('おしえて' in words[j][0] and float(words[j][1]) > 0.8) or ('あと何分' in words[j][0] and float(words[j][1]) > 0.8):
                                siro_timer_set("確認", "", 0)
                                break
                            elif 'キャンセル' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("全部ですか？それとも何件目ですか？")
                                mode = 2  # 聞き耳モードに以降
                                g_parameter = [0, 2, 0, 0, 0]  # 聞き耳を立てる対象を汎用パラメータで指定します。
                                break

                    elif ('名前' in words[i][0] and float(words[i][1]) > 0.6) or ('誰' in words[i][0] and float(words[i][1]) > 0.8):

                        siro_speak("しろちゃんです。")
                        break

                    elif 'ただいま' in words[i][0] and float(words[i][1]) > 0.8:

                        siro_speak("おかえりなさい。")
                        break

                    elif 'いってきます' in words[i][0] and float(words[i][1]) > 0.8:

                        siro_speak("いってらっしゃい")
                        break

                    elif 'おかえり' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_speak("おかえりなさい")
                        break

                    elif 'おはよう' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_speak("おはよーございます。")
                        break

                    elif 'おっす' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_speak("おっす。おらしろちゃん。")
                        break

                    elif 'しずかにして' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_speak("しずかにします。")
                        siro_silent = 1
                        break

                    elif '温度' in words[i][0] and float(words[i][1]) > 0.9:

                        serif = "{0:0.1f}どです。しつどは{1:0.1f}％です。".format(temp, humid)
                        siro_speak(serif)
                        break

                    elif '湿度' in words[i][0] and float(words[i][1]) > 0.9:

                        serif = "現在のしつどは、{0:0.1f}％です。".format(humid)
                        siro_speak(serif)
                        break

                    elif ('今何時' in words[i][0] and float(words[i][1]) > 0.9) or ('今の時刻' in words[i][0] and float(words[i][1]) > 0.9):

                        now = datetime.datetime.now()  # 現在の日付時刻を取得
                        if now.hour <= 12:
                            serif = str(now.hour) + "時" + str(now.minute) + "分です。"
                        else:
                            serif = str(now.hour-12) + "時" + str(now.minute) + "分です。"
                        siro_speak(serif)
                        break

                    elif '何曜' in words[i][0] and float(words[i][1]) > 0.7:
                        now = datetime.datetime.now()  # 現在の日付時刻を取得
                        for j in range(len(words)):
                            if '今日' in words[j][0] and float(words[j][1]) > 0.8:
                                serif = "今日は、" + list_week[now.weekday()] + "です。"
                                siro_speak(serif)
                                break
                            elif '昨日' in words[j][0] and float(words[j][1]) > 0.8:
                                if now.weekday() == 0:
                                    serif = "昨日は、" + list_week[6] + "です。"
                                else:
                                    serif = "昨日は、" + list_week[now.weekday()-1] + "です。"
                                siro_speak(serif)
                                break
                            elif '明日' in words[j][0] and float(words[j][1]) > 0.8:
                                if now.weekday() == 6:
                                    serif = "明日は、" + list_week[0] + "です。"
                                else:
                                    serif = "明日は、" + list_week[now.weekday()+1] + "です。"
                                siro_speak(serif)
                                break
                            elif '明後日' in words[j][0] and float(words[j][1]) > 0.8:
                                if now.weekday() == 5:
                                    serif = "あさっては、" + list_week[0] + "です。"
                                elif now.weekday() == 6:
                                    serif = "あさっては、" + list_week[1] + "です。"
                                else:
                                    serif = "あさっては、" + list_week[now.weekday()+2] + "です。"
                                siro_speak(serif)
                                break
                    elif ('何月' in words[i][0] and float(words[i][1]) > 0.9) or ('何日' in words[i][0] and float(words[i][1]) > 0.9):
                        now = datetime.datetime.now()  # 現在の日付時刻を取得
                        for j in range(len(words)):
                            if '今日' in words[j][0] and float(words[j][1]) > 0.8:
                                serif = "今日は、" + str(now.month) + "月" + str(now.day) + "日、" + list_week[now.weekday()] + "です。"
                                siro_speak(serif)
                                break
                            elif '昨日' in words[j][0] and float(words[j][1]) > 0.8:
                                now = now - datetime.timedelta(days=1)
                                serif = "昨日は、" + str(now.month) + "月" + str(now.day) + "日、" + list_week[now.weekday()] + "です。"
                                siro_speak(serif)
                                break
                            elif '明日' in words[j][0] and float(words[j][1]) > 0.8:
                                now = now + datetime.timedelta(days=1)
                                serif = "明日は、" + str(now.month) + "月" + str(now.day) + "日、" + list_week[now.weekday()] + "です。"
                                siro_speak(serif)
                                break
                            elif '明後日' in words[j][0] and float(words[j][1]) > 0.8:
                                now = now + datetime.timedelta(days=2)
                                serif = "あさっては、" + str(now.month) + "月" + str(now.day) + "日、" + list_week[now.weekday()] + "です。"
                                siro_speak(serif)
                                break

                    elif 'ありがと' in words[i][0] and float(words[i][1]) > 0.6:

                        siro_speak("どういたしまして。")
                        break

                    elif 'どうも' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_sound("ごきげん")
                        break

                    elif 'たすかります' in words[i][0] and float(words[i][1]) > 0.9:

                        siro_speak("とんでもございません。")
                        break

                    elif 'いいこ' in words[i][0] and float(words[i][1]) > 0.8:

                        siro_sound("ごきげん")
                        break

                    elif 'かしこい' in words[i][0] and float(words[i][1]) > 0.7:

                        siro_sound("ごきげん")
                        break

                    elif 'かわいい' in words[i][0] and float(words[i][1]) > 0.7:

                        siro_sound("ごきげん")
                        break

                    elif '天気' in words[i][0] and float(words[i][1]) > 0.9:
                        # わからないことはとりあえずAlexa（スマートスピーカー）に聞きます。
                        # Alexaは優秀なのでSIROの声にも反応してくれます。

                        for j in range(len(words)):
                            if '今日' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("あれくさ。今日の天気はどうですか？")
                                break
                            elif '明日' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("あれくさ。明日の天気はどうですか？")
                                break
                            elif '明後日' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("あれくさ。あさっての天気はどうですか？")
                                break

                    elif '何の日' in words[i][0] and float(words[i][1]) > 0.9:
                        # わからないことはとりあえずAlexa（スマートスピーカー）に聞きます。
                        # Alexaは優秀なのでSIROの声にも反応してくれます。

                        for j in range(len(words)):

                            if '今日' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("あれくさ。今日はなんの日ですか？")
                                break

                            elif '昨日' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("あれくさ。昨日はなんの日ですか？")
                                break

                            elif '明日' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("あれくさ。明日はなんの日ですか？")
                                break

                            elif '明後日' in words[j][0] and float(words[j][1]) > 0.8:
                                siro_speak("あれくさ。あさってはなんの日ですか？")
                                break

                voice_recog = 0
                # 音声認識結果にリアクションしたら音声認識結果読み込み完了フラグを下げます。
                # そうすることでJuliusサーバーからのデータ読み込みが再開します。

            # 応用機能20 ジェスチャー認識へのリアクション **************

            gesture_result = g.return_gesture()
            # ジェスチャー入力結果を読み込みます。

            # if gesture_result == 1:
            # どんなリアクションさせるかは現在作成中です。とりあえず骨子部分だけ完成させました。

            time.sleep(0.5)
            # 処理周期を調整します。

        elif mode == 2:  # 聞き耳モード **********************************
            # 状況別に特定のワードを聞き分けさせたい場合に使用するモードです。
            # 汎用パラメータg_parameter[1]で状況を判別します。
            # g_parameter[1]: 1 : 消去するメッセージの回答待ち
            #                 2 : キャンセルするタイマーの回答待ち

            print("聞き耳モード。")

            # 応用機能19 音声認識へのリアクション ****************
            if voice_recog == 1:
                # 音声認識完了フラグが立っていたら、音声認識結果wordsを確認してリアクションします。

                for i in range(10):
                    print('WORD: {0:s}{1:s}\n'.format(words[i][0], words[i][1]))
                    # 音声認識結果を表示して確認します。

                    if g_parameter[1] == 1:
                        # 応用機能12 メッセージ /消去するメッセージの回答待ち

                        if '全部' in words[i][0] and float(words[i][1]) > 0.9:

                            siro_message("消去", 10)  # 全メッセージを消去
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                        if '一件目' in words[i][0] and float(words[i][1]) > 0.9:
                            siro_message("消去", 1)  # 1件目のメッセージを消去
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                        elif '二件目' in words[i][0] and float(words[i][1]) > 0.9:
                            siro_message("消去", 2)  # 2件目のメッセージを消去
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                        elif '三件目' in words[i][0] and float(words[i][1]) > 0.9:
                            siro_message("消去", 3)  # 3件目のメッセージを消去
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                    if g_parameter[1] == 2:
                        # 応用機能12 タイマー /キャンセルするタイマーの回答待ち

                        if '全部' in words[i][0] and float(words[i][1]) > 0.9:

                            siro_timer_set("キャンセル", "", 10)  # 全タイマーをキャンセル
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                        if '一件目' in words[i][0] and float(words[i][1]) > 0.9:
                            siro_timer_set("キャンセル", "", 1)  # 1件目のタイマーをキャンセル
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                        elif '二件目' in words[i][0] and float(words[i][1]) > 0.9:
                            siro_timer_set("キャンセル", "", 2)  # 2件目のタイマーをキャンセル
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                        elif '三件目' in words[i][0] and float(words[i][1]) > 0.9:
                            siro_timer_set("キャンセル", "", 3)  # 3件目のタイマーをキャンセル
                            mode = 1  # のんびりモードに移行
                            g_parameter = [0, 0, 0, 0, 0]
                            break

                voice_recog = 0
                # 音声認識結果にリアクションしたら音声認識結果読み込み完了フラグを下げます。

            # 一定時間経過したらのんびりモードに戻ります。
            g_parameter[2] = g_parameter[2] + 1  # カウントアップ
            if g_parameter[2] > 30:  # とりあえず約15秒
                mode = 1  # のんびりモードに移行します。
                g_parameter = [0, 0, 0, 0, 0]  # 汎用パラメータ初期化

            time.sleep(0.5)
            # 処理周期を調整します。

        elif mode == 3:  # ラジオモード **********************************

            print("ラジオモード。")

            # 初期処理 ************************************
            if g_parameter[0] == 0:

                siro_speak("ラジオモード。")
                siro_radio("start")
                g_parameter[0] = 1
                # ラジオをつけて初期処理完了フラグを上げます。

            # 応用機能20 ジェスチャー認識へのリアクション *********

            gesture_result = g.return_gesture()
            # ジェスチャー入力結果を読み込みます。

            if gesture_result == 5:  # 上：音量を上げます。

                radio_vol = radio_vol + 10
                if radio_vol > 100:
                    radio_vol = 100

                cmd = "amixer cset numid=3 "+str(radio_vol)+"%"
                subprocess.call(cmd.split())
                # 音量を設定してます。

            elif gesture_result == 6:  # 下:音量を下げます。

                radio_vol = radio_vol - 10
                if radio_vol < 40:
                    radio_vol = 40
                    # 40%を最小に設定しています。

                cmd = "amixer cset numid=3 " + str(radio_vol) + "%"
                subprocess.call(cmd.split())
                # 音量を設定してます。

            elif gesture_result == 3:  # 右：局変更します。

                radiko_ch = radiko_ch + 1
                if radiko_ch == 3:
                    radiko_ch = 0

                siro_radio("stop")
                siro_radio("start")
                # 一旦ラジオを停止し、再び再生することでチャンネルを変更しています。
                # スマートではないですがこうすることでradiko再生シェルスクリプトをそのまま流用できます。

            elif gesture_result == 4:  # 左：局変更します。

                radiko_ch = radiko_ch - 1
                if radiko_ch == -1:
                    radiko_ch = 3

                siro_radio("stop")
                siro_radio("start")

            elif gesture_result == 7:  # 右回り:音量を上げます。

                radio_vol = radio_vol + 10

                if radio_vol > 100:
                    radio_vol = 100

                cmd = "amixer cset numid=3 " + str(radio_vol) + "%"
                subprocess.call(cmd.split())
                # 音量を設定してます。

            elif gesture_result == 8:  # 左回り:音量を下げます。
                radio_vol = radio_vol - 10

                if radio_vol < 40:
                    radio_vol = 40
                    # 40%を最小に設定しています。

                cmd = "amixer cset numid=3 " + str(radio_vol) + "%"
                subprocess.call(cmd.split())
                # 音量を設定してます。

            elif gesture_result == 1:  # 手を近づける：停止 or 再生

                if radio_on == 1:
                    siro_radio("stop")  # ラジオ停止
                else:
                    siro_radio("start")  # ラジオ再生

            elif gesture_result == 9:  # 手をふる。:ラジオモード終了

                if radio_on == 1:
                    siro_radio("stop")  # ラジオ停止
                siro_speak("ラジオモードを終了します。")
                mode = 1  # のんびりモードに移行
                g_parameter = [0, 0, 0, 0, 0]  # 汎用パラメータ初期化

            # 応用機能21 タッチ認識へのリアクション ******************
            if uss_touch == 1:

                # 超音波センサでタッチを検出したらラジオモードを終了してのんびりモードに戻ります。
                if radio_on == 1:
                    siro_radio("stop")  # ラジオ停止
                siro_speak("ラジオモードを終了します。")
                mode = 1  # のんびりモードに移行
                g_parameter = [0, 0, 0, 0, 0]  # 汎用パラメータ初期化

            time.sleep(0.5)
            # 処理周期を調整します。

        elif mode == 4:  # ラジコンモード ********************************

            print("ラジコンモード。")

            # 初期処理 **************************************
            if g_parameter[0] == 0:

                siro_speak("ラジコンモード。")
                siro_motor_speed = 90  # ラジコンモードではモータ速度をUPさせます。
                g_parameter[0] = 1  # 初期処理完了フラグを上げます。

            # 応用機能21 タッチ認識へのリアクション ******************
            if uss_touch == 1:

                # 超音波センサでタッチを検出したらラジコンモードを終了してのんびりモードに戻ります。
                siro_speak("ラジコンモードを終了します。")
                mode = 1  # のんびりモードに移行
                g_parameter = [0, 0, 0, 0, 0]  # 汎用パラメータ初期化

            time.sleep(0.5)
            # 処理周期を調整します。

# 3. 終了処理パート　******************************************************

# Ctrl + C キーボード割り込み **************
except KeyboardInterrupt as reigai:
    print("Ctrl C ........")

# その他例外******************************
except BaseException as reigai:
    print(reigai)  # 例外の中身を表示

# 終了処理 *******************************
finally:

    print("SIRO stopping............")

    GPIO.cleanup()
    # GPIOを初期化します。

    thread_loop = 0
    # サブスレッド終了制御フラグを下げてサブスレッドを終了させます。

    th_facerecog.join()
    print("face recognition stop......")
    th_voicerecog.join()
    print("voice recognition stop......")
    th_uss.join()
    print("Obstacle detection stop.....")
    th_timer.join()
    print("timer stop.....")
    # サブスレッドが正常に終了したことを確認します。

    for i in range(len(personal_data)):
        personal_data[i][3] = str(suki[i])
        # パラメータの編集を反映します。

    f = open("/home/kuro/kurosSIRO/data/personal.csv", 'w')
    dataWriter = csv.writer(f)
    f.write("ID,名前,呼び方,好き％\n")
    dataWriter.writerows(personal_data)
    f.close()
    # 元のCSVファイルに書き込んで編集箇所を反映させます。

    siro_speak("しろ停止します。")
    print("Bye .........")
    input('終了します。エンターキーを押してください。')

# End of program ●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲
