# モジュール読み込み ******************************************************
import time
import RPi.GPIO as GPIO  # GPIO制御
# まず最初にPythonモジュールを読み込んでいます。
# ｔime: 時間に関するPython標準モジュールです。
# RPi.GPIO: Raspberry PiのGPIO（汎用入出力）を制御するためのPythonモジュールです。

# GPIO設定 **************************************************************
GPIO.setmode(GPIO.BCM)  # GPIO番号で指定するモード
# GPIOを使用する際にポート番号で指定するかピン番号で指定するかを設定しています。
# 例えばGPIO2はポート番号で指定すると”２”、ピン番号で指定すると”３”になります。
# ここではポート番号で指定するモードを選択しています。

# 移動 DCモータ制御 モータドライバ DRV8835　**********************************
MOTOR_B_IN1 = 12  # モータ左　ENABLE
MOTOR_B_IN2 = 16  # モータ左　PHASE
MOTOR_A_IN1 = 20  # モータ右　ENABLE
MOTOR_A_IN2 = 21  # モータ右　PHASE
# モータドライバ DRV8835の接続先とGPIOのポート番号を紐づけています。

GPIO.setup(MOTOR_B_IN1, GPIO.OUT)
GPIO.setup(MOTOR_B_IN2, GPIO.OUT)
GPIO.setup(MOTOR_A_IN1, GPIO.OUT)
GPIO.setup(MOTOR_A_IN2, GPIO.OUT)
# GPIOを入力、出力どちらで使用するかを設定しています。ここではすべて出力に設定しています。

MOTOR_B_PWM = GPIO.PWM(MOTOR_B_IN1, 50)  # 50Hz
MOTOR_A_PWM = GPIO.PWM(MOTOR_A_IN1, 50)  # 50Hz
# GPIOの12と20をPWM出力に設定しています。
# パルス周期は50Hzに設定しています。

siro_motor_speed = 50  # Duty比
# PWMのDuty比(0～100)でモーター印加電圧を変化させ回転速度を制御します。
# とりあえず50に設定しています。


def siro_motor(command):
    # SIROを走行させるための関数を定義しています。
    # 引数commandに従い左右のモータの回転速度、回転方向を制御します。

    if (command == "go"):  # まえ
        MOTOR_A_PWM.start(siro_motor_speed)
        MOTOR_B_PWM.start(siro_motor_speed)
        GPIO.output(MOTOR_A_IN2, GPIO.LOW)
        GPIO.output(MOTOR_B_IN2, GPIO.LOW)
    elif (command == "back"):  # うしろ
        MOTOR_A_PWM.start(siro_motor_speed)
        MOTOR_B_PWM.start(siro_motor_speed)
        GPIO.output(MOTOR_A_IN2, GPIO.HIGH)
        GPIO.output(MOTOR_B_IN2, GPIO.HIGH)
    elif (command == "right"):  # 右回転
        MOTOR_A_PWM.start(siro_motor_speed)
        MOTOR_B_PWM.start(siro_motor_speed)
        GPIO.output(MOTOR_A_IN2, GPIO.HIGH)
        GPIO.output(MOTOR_B_IN2, GPIO.LOW)
    elif (command == "left"):  # 左回転
        MOTOR_A_PWM.start(siro_motor_speed)
        MOTOR_B_PWM.start(siro_motor_speed)
        GPIO.output(MOTOR_A_IN2, GPIO.LOW)
        GPIO.output(MOTOR_B_IN2, GPIO.HIGH)
    elif (command == "stop"):  # 停止
        MOTOR_A_PWM.stop()
        MOTOR_B_PWM.stop()
        GPIO.output(MOTOR_A_IN2, GPIO.LOW)
        GPIO.output(MOTOR_B_IN2, GPIO.LOW)


# ここまでが初期設定パートでここからメイン処理パートです。
# メイン処理 *************************************************************
try:  # try,except構文で例外処理を行っています。
    while True:  # While文で繰り返し処理を行っています。

        # 前進、後進、左回転、右回転を繰り返します。
        siro_motor("go")
        print("go")
        time.sleep(3)

        siro_motor("back")
        print("back")
        time.sleep(3)

        siro_motor("left")
        print("left")
        time.sleep(3)

        siro_motor("right")
        print("right")
        time.sleep(3)

        siro_motor("stop")
        print("stop")

        time.sleep(3)
        # 3秒毎に実行します。

# 以降は終了処理パートです。
# 終了処理　**************************************************************
# 例外の一つCtrl +C (キーボード割り込み)で終了処理に移行します。
except KeyboardInterrupt as reigai:
    print("Ctrl C ........")

# ここから終了処理を記載するパートです。
finally:
    print("SIRO stopping............")
    GPIO.cleanup()  # GPIO初期化

# End of program ●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■
# プログラムの最後です。
