# モジュール読み込み ******************************************************
import cv2  # openCV
# cv2: OpenCVのPythonモジュールです。

# 顔認識 ****************************************************************
face_cascade = \
    cv2.CascadeClassifier('/home/kuro/kurosSIRO/haarcascades/haarcascade_frontalface_alt.xml')
# 顔認識学習モデル（カスケード分類器）をファイル名で指定して読み込んでいます。

cap = cv2.VideoCapture(0)
# クラスのインスタンス生成

cap.set(3, 640)  # 画像サイズ WIDTH
cap.set(4, 480)  # 画像サイズ HEIGHT
# カメラの解像度を設定しています。
# 解像度を大きくすると画像が鮮明になりますが処理動作が遅くなります。

cap.set(cv2.CAP_PROP_FPS, 30)
# FPSを設定しています。
# 使用したUSBカメラはFPS30で固定なので設定してもあまり意味がないのですが一応明示しています。

cap.set(cv2.CAP_PROP_BUFFERSIZE, 1)
# バッファサイズを最小の１に設定しています。
# こうすることでバッファ分のフレームの遅延がなくなります。

minW = 50
minH = 50
# 認識可能な顔のサイズを設定しています。
# 大きくすると遠くの顔が認識できなくなりますが誤認識が少なくなります。
# 逆に小さくすると遠くの顔も認識できる様になりますが壁の模様を顔と間違える等の誤認識が増えます。

# メイン処理 *************************************************************
face_id = input('ID No.を入力してエンターを押してください。\n')
# ID番号を入力します。筆者の場合は 0: 自分　1：妻　2：長男　3：次男　 としています。

print("カメラを見てお待ちください。\n")
count = 0  # カウンター初期化

try:
    while True:

        ret, frame = cap.read()
        # カメラ画像を取得しています。

        grayimg = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        # 顔認識を行う前に画像をモノクロ処理して情報量を減らしています。

        facerect = face_cascade.detectMultiScale(grayimg, minSize=(minW, minH))
        # 顔認識を行っています。

        if len(facerect) > 0:
        # 顔を見つけたら顔画像データを保存します。

            count += 1  # カウントアップ
            print(count, " 枚目")

            for (x, y, w, h) in facerect:
                cv2.rectangle(frame, (x, y), (x+w, y+h), (255, 0, 0), 2)  # 顔検出箇所を□で囲む
                cv2.imwrite("/home/kuro/kurosSIRO/data/face/User."
                            + str(face_id) + '.' + str(count) + ".jpg", grayimg[y:y+h, x:x+w])
                # カメラ画像から顔部分を抽出しファイル名を指定してｊpgで保存しています。

        if (count >= 100):
            # 顔画像を100枚取得したらwhileループから抜けます。
            print("終了します\n")
            break

        cv2.imshow('camera capture', frame)
        # カメラ画像をウインドウに表示させています。

        cv2.waitKey(100)
        # 処理周期を100msecに設定しています。

# 終了処理　*************************************************************
# Ctrl + C キーボード割り込み
except KeyboardInterrupt as reigai:
    print("Ctrl C ........")

finally:  # 終了処理
    print("SIRO stopping............")
    cap.release()
    # OpenＣＶを終了しカメラデバイスを解放します。
    cv2.destroyAllWindows()
    # カメラ画像表示ウインドウを閉じます。

# End of program ●▲■●▲■●▲■●▲■■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■
