# モジュール読み込み ******************************************************
import cv2  # openCV
import os
import numpy as np
from PIL import Image
# cv2: OpenCVのPythonモジュールです。
# os: os機能に関するPython標準モジュールです。
# numpy: 行列計算を処理するためのPythonモジュールです。科学技術計算や機械学習などに使用されます。
# PIL: 画像処理用のPythonモジュールです。

# 顔認識 ****************************************************************
path = '/home/kuro/kurosSIRO/data/face'
# 顔画像データを保存したフォルダを指定してます。

face_cascade = \
        cv2.CascadeClassifier('/home/kuro/kurosSIRO/haarcascades/haarcascade_frontalface_alt.xml')
# 顔認識学習モデル（カスケード分類器）をファイル名で指定して読み込んでいます。

recognizer = cv2.face.LBPHFaceRecognizer_create()
# 顔判別用機械学習クラスのインスタンス生成


# 指定したフォルダの画像を読み込んでrecognizer.trainに渡す変数を生成する関数です。
# ネット上のOpenCV関連のサンプルプログラムを流用しましたので詳細は割愛します。
def getImagesAndLabels(path):
    imagePaths = [os.path.join(path, f) for f in os.listdir(path)]
    faceSamples = []
    ids = []

    for imagePath in imagePaths:

        # PIL_img = Image.open(imagePath).convert('L')  # convert it to grayscale 
        PIL_img = Image.open(imagePath)
        img_numpy = np.array(PIL_img, 'uint8')

        id = int(os.path.split(imagePath)[-1].split(".")[1])
        faces = face_cascade.detectMultiScale(img_numpy)

        for (x, y, w, h) in faces:
            faceSamples.append(img_numpy[y:y+h, x:x+w])
            ids.append(id)

    return faceSamples, ids


# メイン処理  ************************************************************

print("顔画像の学習を始めます。数秒かかります。...")

faces, ids = getImagesAndLabels(path)
recognizer.train(faces, np.array(ids))
recognizer.write('/home/kuro/kurosSIRO/trainer.yml')
# 機械学習を行い、その結果をtrainer.ymlというファイル名で出力します。

print("\n {0} 個の顔を学習しました。\nプログラムを終了します。".format(len(np.unique(ids))))

# End of program ●▲■●▲■●▲■●▲■■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■
