# モジュール読み込み ******************************************************
import time
import subprocess
import socket
import threading
# ｔime: 時間に関するPython標準モジュールです。
# subproces:プロセス管理に関するPython標準モジュールです。
# socket: socket通信に関するPython標準モジュールです。
# threading: マルチスレッド処理に関するPython標準モジュールです。

# 音声認識　Julius *******************************************************
print("Julius server start.....")
cmd = "bash /home/kuro/kurosSIRO/julius/julius_module.sh"
subprocess.Popen(cmd.split())
# Juliusをモジュールモード（音声認識サーバー）で起動します。

time.sleep(5)
# Juliusの起動を待ちます。5秒は適当です。

print("Conecting Julius server.....",)
client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
HOST = "localhost"  # JuliusサーバーのIPアドレス,自分自身なのでlocalhost
PORT_julius = 10500  # juliusサーバーの接続ポート番号
client.connect((HOST, PORT_julius))
# IPアドレスとポート番号を指定してクライアントとしてJuliusサーバーに接続します。

words = [['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', '']]
# 音声認識結果を格納するためのリストです。

bufsize = 1024
# Juliusサーバーからデータを読み込む際の1回あたりのデータサイズ[byte]です。

voice_recog = 0
# 音声認識結果読み込み完了フラグです。
# フラグが立ったらメインスレッドで音声認識結果にリアクションします。

thread_loop = 1
# サブスレッドの終了を制御するためのフラグです。


def julius_crient():
    # Juliusサーバーから音声認識結果を読み込む関数です。

    global thread_loop, words, voice_recog

    while (thread_loop):
        # メイン処理終了時にthread_loopを0にしてループから脱出させてサブスレッドを終了させます。

        if (voice_recog == 0):
            # 音声認識結果読み込み完了フラグが下がっていたらJuliusからデータの読み込みを開始します。

            julius_data = ""
            words = [['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', ''], ['', '']]
            counter = 0
            # まずは初期化

            while (julius_data.find("</RECOGOUT>") == -1 and julius_data.find("<RECOGFAIL/>") == -1):
                julius_data = julius_data + client.recv(1024).decode('utf-8')
                # Juliusからデータを読み込みます。データの中に音声認識完了を示すキーワード</RECOGOUT>
                # 若しくは音声認識失敗を示すキーワード<RECOGFAIL/>のどちらかが現れたら読み込みを終了します。

            for line in julius_data.split('\n'):  # １行ごとに探す
                index = line.find('WORD="')
                if index != -1:  # WORDが見つかったら以下を実施
                    words[counter][0] = line[index + 6:line.find('"', index+6)]  # WORDを格納
                    index = line.find('CM="')  # CMをさがす。
                    words[counter][1] = line[index + 4:line.find('"', index+4)]  # CMを格納
                    counter = counter + 1  # カウントアップ
            # Juliusサーバーから取得したデータ（XML形式）の中から、認識した単語”WORD”とその信頼度”CM”の値を抽出しています。

            voice_recog = 1
            # 音声認識結果の読み込みが完了したらフラグを立てます。

            print(julius_data)
            # Juliusサーバーから取得した生データの中身を確認します。

        time.sleep(0.1)
        # 処理周期を調整しています。


th_voicerecog = threading.Thread(target=julius_crient, daemon=True)
th_voicerecog.start()  # サブスレッド開始
print("Voice recognition start.....",)
# 音声認識処理は音声入力待ちが発生するのでメイン処理に影響を与えないようサブスレッドで処理します。

# メイン処理 *************************************************************
try:
    while True:

        # 音声認識 *********************
        if (voice_recog == 1):
            # 音声認識結果読み込み完了フラグが立っていたら、音声認識結果wordsを確認してリアクションします。

            for i in range(10):
                print('WORD: {0:s}{1:s}\n'.format(words[i][0], words[i][1]))
                # 音声認識結果を表示して確認します。

            for i in range(len(words)):
                if ('おはよう' in words[i][0] and float(words[i][1]) > 0.9):
                    # 音声認識結果に”おはよう”というワードが見つかった場合のリアクションです。
                    print("おはようございます。")

                # このブロックにif文を追加していくことで、ワードへのリアクションを追加することができます。

            voice_recog = 0
            # 音声認識結果にリアクションしたら音声認識結果読み込み完了フラグを下げます。
            # そうすることでサブスレッドでのJuliusからのデータ読み込みが再開します。

        time.sleep(1)
        # 処理周期を調整しています。

# 終了処理　**************************************************************
# Ctrl + C キーボード割り込み
except KeyboardInterrupt as reigai:
    print("Ctrl C ........")

finally:  # 終了処理
    print("SIRO stopping............")
    print("Julius_voicerecognition stop......")
    client.close()
    # Juliusサーバとの通信を終了します。

    thread_loop = 0
    # サブスレッド終了フラグを下げてサブスレッド（音声認識処理）を終了させます。

    th_voicerecog.join()
    # サブスレッドの終了を待ちます。

# End of program ●▲■●▲■●▲■●▲■■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲■●▲
